@@ -5,6 +5,11 @@
 // NULL
 #include "stddef.h"
 
+// suppress MSVC C4244: conversion from uECC_word_t to int
+#ifdef _MSC_VER
+#pragma warning( disable : 4244 )
+#endif
+
 #ifndef uECC_PLATFORM
     #if __AVR__
         #define uECC_PLATFORM uECC_avr
@@ -375,10 +380,14 @@ static const uECC_word_t curve_n[uECC_N_WORDS] = uECC_CONCAT(Curve_N_, uECC_CURV
 static void vli_clear(uECC_word_t *vli);
 static uECC_word_t vli_isZero(const uECC_word_t *vli);
 static uECC_word_t vli_testBit(const uECC_word_t *vli, bitcount_t bit);
+#ifdef ENABLE_MICRO_ECC_ECDSA
 static bitcount_t vli_numBits(const uECC_word_t *vli, wordcount_t max_words);
+#endif
 static void vli_set(uECC_word_t *dest, const uECC_word_t *src);
 static cmpresult_t vli_cmp(const uECC_word_t *left, const uECC_word_t *right);
+#ifdef ENABLE_MICRO_ECC_ECDSA
 static cmpresult_t vli_equal(const uECC_word_t *left, const uECC_word_t *right);
+#endif
 static void vli_rshift1(uECC_word_t *vli);
 static uECC_word_t vli_add(uECC_word_t *result,
                            const uECC_word_t *left,
@@ -463,6 +472,8 @@ static int default_RNG(uint8_t *dest, unsigned size) {
 #else /* Some other platform */
 
 static int default_RNG(uint8_t *dest, unsigned size) {
+    (void) dest;
+    (void) size;
     return 0;
 }
 
@@ -514,6 +525,8 @@ static uECC_word_t vli_testBit(const uECC_word_t *vli, bitcount_t bit) {
 }
 #endif
 
+#ifdef ENABLE_MICO_ECC_ECDSA
+
 /* Counts the number of words in vli. */
 #if !asm_numBits
 static wordcount_t vli_numDigits(const uECC_word_t *vli, wordcount_t max_words) {
@@ -543,6 +556,9 @@ static bitcount_t vli_numBits(const uECC_word_t *vli, wordcount_t max_words) {
 
     return (((bitcount_t)(num_digits - 1) << uECC_WORD_BITS_SHIFT) + i);
 }
+
+#endif /* ENABLE_MICO_ECC_ECDSA */
+
 #endif /* !asm_numBits */
 
 /* Sets dest = src. */
@@ -570,6 +586,8 @@ static cmpresult_t vli_cmp(const uECC_word_t *left, const uECC_word_t *right) {
 }
 #endif
 
+#ifdef ENABLE_MICRO_ECC_ECDSA
+
 static cmpresult_t vli_equal(const uECC_word_t *left, const uECC_word_t *right) {
     uECC_word_t result = 0;
     swordcount_t i;
@@ -579,16 +597,19 @@ static cmpresult_t vli_equal(const uECC_word_t *left, const uECC_word_t *right)
     return (result == 0);
 }
 
+#endif
+
 /* Computes vli = vli >> 1. */
 #if !asm_rshift1
 static void vli_rshift1(uECC_word_t *vli) {
     uECC_word_t *end = vli;
     uECC_word_t carry = 0;
+    uECC_word_t *vli_ = vli;
 
-    vli += uECC_WORDS;
-    while (vli-- > end) {
-        uECC_word_t temp = *vli;
-        *vli = (temp >> 1) | carry;
+    vli_ += uECC_WORDS;
+    while (vli_-- > end) {
+        uECC_word_t temp = *vli_;
+        *vli_ = (temp >> 1) | carry;
         carry = temp << (uECC_WORD_BITS - 1);
     }
 }
@@ -1816,6 +1837,8 @@ static int EccPoint_compute_public_key(EccPoint *result, uECC_word_t *private) {
     return 1;
 }
 
+#ifdef ENABLE_MICRO_ECC_COMPRESSION
+
 #if uECC_CURVE == uECC_secp224r1
 
 /* Routine 3.2.4 RS;  from http://www.nsa.gov/ia/_files/nist-routines.pdf */
@@ -1952,6 +1975,9 @@ static void mod_sqrt(uECC_word_t *a) {
 }
 #endif /* uECC_CURVE */
 
+#endif /* ENABLE_MICRO_ECC_COMPRESSION */
+
+
 #if uECC_WORD_SIZE == 1
 
 static void vli_nativeToBytes(uint8_t * RESTRICT dest, const uint8_t * RESTRICT src) {
@@ -2073,6 +2099,8 @@ int uECC_shared_secret(const uint8_t public_key[uECC_BYTES*2],
     return !EccPoint_isZero(&product);
 }
 
+#ifdef ENABLE_MICRO_ECC_COMPRESSION
+
 void uECC_compress(const uint8_t public_key[uECC_BYTES*2], uint8_t compressed[uECC_BYTES+1]) {
     wordcount_t i;
     for (i = 0; i < uECC_BYTES; ++i) {
@@ -2081,6 +2109,8 @@ void uECC_compress(const uint8_t public_key[uECC_BYTES*2], uint8_t compressed[uE
     compressed[0] = 2 + (public_key[uECC_BYTES * 2 - 1] & 0x01);
 }
 
+#endif
+
 /* Computes result = x^3 + ax + b. result must not overlap x. */
 static void curve_x_side(uECC_word_t * RESTRICT result, const uECC_word_t * RESTRICT x) {
     static const uECC_word_t curve_b[uECC_WORDS] = uECC_CONCAT(Curve_B_, uECC_CURVE);
@@ -2098,6 +2128,8 @@ static void curve_x_side(uECC_word_t * RESTRICT result, const uECC_word_t * REST
 #endif
 }
 
+#ifdef ENABLE_MICRO_ECC_COMPRESSION
+
 void uECC_decompress(const uint8_t compressed[uECC_BYTES+1], uint8_t public_key[uECC_BYTES*2]) {
     EccPoint point;
     vli_bytesToNative(point.x, compressed + 1);
@@ -2112,6 +2144,8 @@ void uECC_decompress(const uint8_t compressed[uECC_BYTES+1], uint8_t public_key[
     vli_nativeToBytes(public_key + uECC_BYTES, point.y);
 }
 
+#endif /* ENABLE_MICRO_ECC_COMPRESSION */
+
 int uECC_valid_public_key(const uint8_t public_key[uECC_BYTES*2]) {
     uECC_word_t tmp1[uECC_WORDS];
     uECC_word_t tmp2[uECC_WORDS];
@@ -2163,6 +2197,8 @@ int uECC_curve(void) {
 
 /* -------- ECDSA code -------- */
 
+#ifdef ENABLE_MICRO_ECC_ECDSA
+
 #if (uECC_CURVE == uECC_secp160r1)
 static void vli_clear_n(uECC_word_t *vli) {
     vli_clear(vli);
@@ -2745,3 +2781,5 @@ int uECC_verify(const uint8_t public_key[uECC_BYTES*2],
     /* Accept only if v == r. */
     return vli_equal(rx, r);
 }
+
+#endif
