@@ -21,8 +21,8 @@
  * THIS SOFTWARE IS PROVIDED BY BLUEKITCHEN GMBH AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATTHIAS
- * RINGWALD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL BLUEKITCHEN
+ * GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
@@ -43,7 +43,7 @@
  */
 
 // *****************************************************************************
-/* EXAMPLE_START(hfp_hs_demo): HFP Hands-Free (HF) Demo
+/* EXAMPLE_START(hfp_hs_demo): HFP HF - Hands-Free
  *
  * @text This  HFP Hands-Free example demonstrates how to receive 
  * an output from a remote HFP audio gateway (AG), and, 
@@ -68,7 +68,7 @@ const char hfp_hf_service_name[] = "HFP HF Demo";
 
 #ifdef HAVE_BTSTACK_STDIN
 // static const char * device_addr_string = "6C:72:E7:10:22:EE";
-static const char * device_addr_string = "54:E4:3A:26:A2:39";
+static const char * device_addr_string = "00:02:72:DC:31:C1";
 #endif
 
 static bd_addr_t device_addr;
@@ -88,7 +88,7 @@ static uint8_t codecs[] = {HFP_CODEC_CVSD};
 static uint16_t indicators[1] = {0x01};
 static uint8_t  negotiated_codec = HFP_CODEC_CVSD;
 static btstack_packet_callback_registration_t hci_event_callback_registration;
-char cmd;
+static char cmd;
 
 static void dump_supported_codecs(void){
     unsigned int i;
@@ -106,6 +106,9 @@ static void dump_supported_codecs(void){
                     mSBC_skipped = 1;
                 }
                 break;
+            default:
+                btstack_assert(false);
+                break;
         }
     }
     printf("\n");
@@ -140,15 +143,16 @@ static void show_usage(void){
     printf("U - end active call and accept other call' (TWC 1)\n");
     printf("v - Swap active call call (TWC 2)          | V - Join held call (TWC 3)\n");
     printf("w - Connect calls (TWC 4)                  | W - redial\n");
+    printf("m - deactivate echo canceling and noise reduction\n");
     printf("c/C - disable/enable registration status update for all AG indicators\n");
     printf("e/E - disable/enable reporting of the extended AG error result code\n");
     printf("k/K - deactivate/activate call waiting notification\n");
     printf("l/L - deactivate/activate calling line notification\n");
-    printf("m/M - deactivate/activate echo canceling and noise reduction\n");
     printf("n/N - deactivate/activate voice recognition\n");
+    
     printf("0123456789#*-+ - send DTMF dial tones\n");
     printf("x - request phone number for voice tag     | X - current call status (ECS)\n");
-    printf("y - release call with index 2 (ECC)        | Y - private consulation with call 2(ECC)\n");
+    printf("y - release call with index 2 (ECC)        | Y - private consultation with call 2(ECC)\n");
     printf("[ - Query Response and Hold status (RHH ?) | ] - Place call in a response and held state(RHH 0)\n");
     printf("{ - Accept held call(RHH 1)                | } - Reject held call(RHH 2)\n");
     printf("? - Query Subscriber Number (NUM)\n");
@@ -156,12 +160,22 @@ static void show_usage(void){
     printf("\n");
 }
 
+static void report_status(uint8_t status, const char * message){
+    if (status != ERROR_CODE_SUCCESS){
+        printf("%s command failed, status 0x%02x\n", message, status);
+    } else {
+        printf("%s command successful\n", message);
+    }
+}
+
 static void stdin_process(char c){
+    uint8_t status = ERROR_CODE_SUCCESS;
+
     cmd = c;    // used in packet handler
 
     if (cmd >= '0' && cmd <= '9'){
         printf("DTMF Code: %c\n", cmd);
-        hfp_hf_send_dtmf_code(acl_handle, cmd);
+        status = hfp_hf_send_dtmf_code(acl_handle, cmd);
         return;
     }
 
@@ -172,77 +186,77 @@ static void stdin_process(char c){
         case '*':
             log_info("USER:\'%c\'", cmd);
             printf("DTMF Code: %c\n", cmd);
-            hfp_hf_send_dtmf_code(acl_handle, cmd);
+            status = hfp_hf_send_dtmf_code(acl_handle, cmd);
             break;
         case 'a':
             log_info("USER:\'%c\'", cmd);
             printf("Establish Service level connection to device with Bluetooth address %s...\n", bd_addr_to_str(device_addr));
-            hfp_hf_establish_service_level_connection(device_addr);
+            status = hfp_hf_establish_service_level_connection(device_addr);
             break;
         case 'A':
             log_info("USER:\'%c\'", cmd);
             printf("Release Service level connection.\n");
-            hfp_hf_release_service_level_connection(acl_handle);
+            status = hfp_hf_release_service_level_connection(acl_handle);
             break;
         case 'b':
             log_info("USER:\'%c\'", cmd);
             printf("Establish Audio connection to device with Bluetooth address %s...\n", bd_addr_to_str(device_addr));
-            hfp_hf_establish_audio_connection(acl_handle);
+            status = hfp_hf_establish_audio_connection(acl_handle);
             break;
         case 'B':
             log_info("USER:\'%c\'", cmd);
             printf("Release Audio service level connection.\n");
-            hfp_hf_release_audio_connection(acl_handle);
+            status = hfp_hf_release_audio_connection(acl_handle);
             break;
         case 'C':
             log_info("USER:\'%c\'", cmd);
             printf("Enable registration status update for all AG indicators.\n");
-            hfp_hf_enable_status_update_for_all_ag_indicators(acl_handle);
+            status = hfp_hf_enable_status_update_for_all_ag_indicators(acl_handle);
             break;
         case 'c':
             log_info("USER:\'%c\'", cmd);
             printf("Disable registration status update for all AG indicators.\n");
-            hfp_hf_disable_status_update_for_all_ag_indicators(acl_handle);
+            status = hfp_hf_disable_status_update_for_all_ag_indicators(acl_handle);
             break;
         case 'D':
             log_info("USER:\'%c\'", cmd);
             printf("Set HFP AG registration status update for individual indicators (0111111).\n");
-            hfp_hf_set_status_update_for_individual_ag_indicators(acl_handle, 63);
+            status = hfp_hf_set_status_update_for_individual_ag_indicators(acl_handle, 63);
             break;
         case 'd':
             log_info("USER:\'%c\'", cmd);
             printf("Query network operator.\n");
-            hfp_hf_query_operator_selection(acl_handle);
+            status = hfp_hf_query_operator_selection(acl_handle);
             break;
         case 'E':
             log_info("USER:\'%c\'", cmd);
             printf("Enable reporting of the extended AG error result code.\n");
-            hfp_hf_enable_report_extended_audio_gateway_error_result_code(acl_handle);
+            status = hfp_hf_enable_report_extended_audio_gateway_error_result_code(acl_handle);
             break;
         case 'e':
             log_info("USER:\'%c\'", cmd);
             printf("Disable reporting of the extended AG error result code.\n");
-            hfp_hf_disable_report_extended_audio_gateway_error_result_code(acl_handle);
+            status = hfp_hf_disable_report_extended_audio_gateway_error_result_code(acl_handle);
             break;
         case 'f':
             log_info("USER:\'%c\'", cmd);
             printf("Answer incoming call.\n");
-            hfp_hf_answer_incoming_call(acl_handle);
+            status = hfp_hf_answer_incoming_call(acl_handle);
             break;
         case 'F':
             log_info("USER:\'%c\'", cmd);
             printf("Hangup call.\n");
-            hfp_hf_terminate_call(acl_handle);
+            status = hfp_hf_terminate_call(acl_handle);
             break;
         case 'G':
             log_info("USER:\'%c\'", cmd);
             printf("Reject incoming call.\n");
-            hfp_hf_reject_incoming_call(acl_handle);
+            status = hfp_hf_reject_incoming_call(acl_handle);
             break;
         case 'g':
             log_info("USER:\'%c\'", cmd);
             printf("Query operator.\n");
-            hfp_hf_query_operator_selection(acl_handle);
+            status = hfp_hf_query_operator_selection(acl_handle);
             break;
         case 't':
             log_info("USER:\'%c\'", cmd);
@@ -252,193 +266,193 @@ static void stdin_process(char c){
         case 'i':
             log_info("USER:\'%c\'", cmd);
             printf("Dial 1234567\n");
-            hfp_hf_dial_number(acl_handle, "1234567");
+            status = hfp_hf_dial_number(acl_handle, "1234567");
             break;
         case 'I':
             log_info("USER:\'%c\'", cmd);
             printf("Dial 7654321\n");
-            hfp_hf_dial_number(acl_handle, "7654321");
+            status = hfp_hf_dial_number(acl_handle, "7654321");
             break;
         case 'j':
             log_info("USER:\'%c\'", cmd);
             printf("Dial #1\n");
-            hfp_hf_dial_memory(acl_handle,1);
+            status = hfp_hf_dial_memory(acl_handle,1);
             break;
         case 'J':
             log_info("USER:\'%c\'", cmd);
             printf("Dial #99\n");
-            hfp_hf_dial_memory(acl_handle,99);
+            status = hfp_hf_dial_memory(acl_handle,99);
             break;
         case 'k':
             log_info("USER:\'%c\'", cmd);
             printf("Deactivate call waiting notification\n");
-            hfp_hf_deactivate_call_waiting_notification(acl_handle);
+            status = hfp_hf_deactivate_call_waiting_notification(acl_handle);
             break;
         case 'K':
             log_info("USER:\'%c\'", cmd);
             printf("Activate call waiting notification\n");
-            hfp_hf_activate_call_waiting_notification(acl_handle);
+            status = hfp_hf_activate_call_waiting_notification(acl_handle);
             break;
         case 'l':
             log_info("USER:\'%c\'", cmd);
             printf("Deactivate calling line notification\n");
-            hfp_hf_deactivate_calling_line_notification(acl_handle);
+            status = hfp_hf_deactivate_calling_line_notification(acl_handle);
             break;
         case 'L':
             log_info("USER:\'%c\'", cmd);
             printf("Activate calling line notification\n");
-            hfp_hf_activate_calling_line_notification(acl_handle);
+            status = hfp_hf_activate_calling_line_notification(acl_handle);
             break;
         case 'm':
             log_info("USER:\'%c\'", cmd);
             printf("Deactivate echo canceling and noise reduction\n");
-            hfp_hf_deactivate_echo_canceling_and_noise_reduction(acl_handle);
-            break;
-        case 'M':
-            log_info("USER:\'%c\'", cmd);
-            printf("Activate echo canceling and noise reduction\n");
-            hfp_hf_activate_echo_canceling_and_noise_reduction(acl_handle);
+            status = hfp_hf_deactivate_echo_canceling_and_noise_reduction(acl_handle);
             break;
         case 'n':
             log_info("USER:\'%c\'", cmd);
             printf("Deactivate voice recognition\n");
-            hfp_hf_deactivate_voice_recognition_notification(acl_handle);
+            status = hfp_hf_deactivate_voice_recognition(acl_handle);
             break;
         case 'N':
             log_info("USER:\'%c\'", cmd);
             printf("Activate voice recognition %s\n", bd_addr_to_str(device_addr));
-            hfp_hf_activate_voice_recognition_notification(acl_handle);
+            status = hfp_hf_activate_voice_recognition(acl_handle);
             break;
         case 'o':
             log_info("USER:\'%c\'", cmd);
             printf("Set speaker gain to 0 (minimum)\n");
-            hfp_hf_set_speaker_gain(acl_handle, 0);
+            status = hfp_hf_set_speaker_gain(acl_handle, 0);
             break;
         case 'O':
             log_info("USER:\'%c\'", cmd);
             printf("Set speaker gain to 9 (default)\n");
-            hfp_hf_set_speaker_gain(acl_handle, 9);
+            status = hfp_hf_set_speaker_gain(acl_handle, 9);
             break;
         case 'p':
             log_info("USER:\'%c\'", cmd);
             printf("Set speaker gain to 12 (higher)\n");
-            hfp_hf_set_speaker_gain(acl_handle, 12);
+            status = hfp_hf_set_speaker_gain(acl_handle, 12);
             break;
         case 'P':
             log_info("USER:\'%c\'", cmd);
             printf("Set speaker gain to 15 (maximum)\n");
-            hfp_hf_set_speaker_gain(acl_handle, 15);
+            status = hfp_hf_set_speaker_gain(acl_handle, 15);
             break;
         case 'q':
             log_info("USER:\'%c\'", cmd);
             printf("Set microphone gain to 0\n");
-            hfp_hf_set_microphone_gain(acl_handle, 0);
+            status = hfp_hf_set_microphone_gain(acl_handle, 0);
             break;
         case 'Q':
             log_info("USER:\'%c\'", cmd);
             printf("Set microphone gain to 9\n");
-            hfp_hf_set_microphone_gain(acl_handle, 9);
+            status = hfp_hf_set_microphone_gain(acl_handle, 9);
             break;
         case 's':
             log_info("USER:\'%c\'", cmd);
             printf("Set microphone gain to 12\n");
-            hfp_hf_set_microphone_gain(acl_handle, 12);
+            status = hfp_hf_set_microphone_gain(acl_handle, 12);
             break;
         case 'S':
             log_info("USER:\'%c\'", cmd);
             printf("Set microphone gain to 15\n");
-            hfp_hf_set_microphone_gain(acl_handle, 15);
+            status = hfp_hf_set_microphone_gain(acl_handle, 15);
             break;
         case 'u':
             log_info("USER:\'%c\'", cmd);
             printf("Send 'user busy' (Three-Way Call 0)\n");
-            hfp_hf_user_busy(acl_handle);
+            status = hfp_hf_user_busy(acl_handle);
             break;
         case 'U':
             log_info("USER:\'%c\'", cmd);
             printf("End active call and accept waiting/held call (Three-Way Call 1)\n");
-            hfp_hf_end_active_and_accept_other(acl_handle);
+            status = hfp_hf_end_active_and_accept_other(acl_handle);
             break;
         case 'v':
             log_info("USER:\'%c\'", cmd);
             printf("Swap active call and hold/waiting call (Three-Way Call 2)\n");
-            hfp_hf_swap_calls(acl_handle);
+            status = hfp_hf_swap_calls(acl_handle);
             break;
         case 'V':
             log_info("USER:\'%c\'", cmd);
             printf("Join hold call (Three-Way Call 3)\n");
-            hfp_hf_join_held_call(acl_handle);
+            status = hfp_hf_join_held_call(acl_handle);
             break;
         case 'w':
             log_info("USER:\'%c\'", cmd);
             printf("Connect calls (Three-Way Call 4)\n");
-            hfp_hf_connect_calls(acl_handle);
+            status = hfp_hf_connect_calls(acl_handle);
             break;
         case 'W':
             log_info("USER:\'%c\'", cmd);
             printf("Redial\n");
-            hfp_hf_redial_last_number(acl_handle);
+            status = hfp_hf_redial_last_number(acl_handle);
             break;
         case 'x':
             log_info("USER:\'%c\'", cmd);
             printf("Request phone number for voice tag\n");
-            hfp_hf_request_phone_number_for_voice_tag(acl_handle);
+            status = hfp_hf_request_phone_number_for_voice_tag(acl_handle);
             break;
         case 'X':
             log_info("USER:\'%c\'", cmd);
             printf("Query current call status\n");
-            hfp_hf_query_current_call_status(acl_handle);
+            status = hfp_hf_query_current_call_status(acl_handle);
             break;
         case 'y':
             log_info("USER:\'%c\'", cmd);
             printf("Release call with index 2\n");
-            hfp_hf_release_call_with_index(acl_handle, 2);
+            status = hfp_hf_release_call_with_index(acl_handle, 2);
             break;
         case 'Y':
             log_info("USER:\'%c\'", cmd);
-            printf("Private consulation with call 2\n");
-            hfp_hf_private_consultation_with_call(acl_handle, 2);
+            printf("Private consultation with call 2\n");
+            status = hfp_hf_private_consultation_with_call(acl_handle, 2);
             break;
         case '[':
             log_info("USER:\'%c\'", cmd);
             printf("Query Response and Hold status (RHH ?)\n");
-            hfp_hf_rrh_query_status(acl_handle);
+            status = hfp_hf_rrh_query_status(acl_handle);
             break;
         case ']':
             log_info("USER:\'%c\'", cmd);
             printf("Place call in a response and held state (RHH 0)\n");
-            hfp_hf_rrh_hold_call(acl_handle);
+            status = hfp_hf_rrh_hold_call(acl_handle);
            break;
         case '{':
             log_info("USER:\'%c\'", cmd);
             printf("Accept held call (RHH 1)\n");
-            hfp_hf_rrh_accept_held_call(acl_handle);
+            status = hfp_hf_rrh_accept_held_call(acl_handle);
             break;
         case '}':
             log_info("USER:\'%c\'", cmd);
             printf("Reject held call (RHH 2)\n");
-            hfp_hf_rrh_reject_held_call(acl_handle);
+            status = hfp_hf_rrh_reject_held_call(acl_handle);
             break;
         case '?':
             log_info("USER:\'%c\'", cmd);
             printf("Query Subscriber Number\n");
-            hfp_hf_query_subscriber_number(acl_handle);
+            status = hfp_hf_query_subscriber_number(acl_handle);
             break;
         case '!':
             log_info("USER:\'%c\'", cmd);
             printf("Update HF indicator with assigned number 1 (HFI)\n");
-            hfp_hf_set_hf_indicator(acl_handle, 1, 1);
+            status = hfp_hf_set_hf_indicator(acl_handle, 1, 1);
             break;
         default:
             show_usage();
             break;
     }
+
+    if (status != ERROR_CODE_SUCCESS){
+        printf("Could not perform command, status 0x%02x\n", status);
+    }
 }
 #endif
 
 static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * event, uint16_t event_size){
     UNUSED(channel);
-    
+    uint8_t status;
+
     switch (packet_type){
 
         case HCI_SCO_DATA_PACKET:
@@ -447,21 +461,25 @@ static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * even
             break;
 
         case HCI_EVENT_PACKET:
-            switch (event[0]){
+            switch (hci_event_packet_get_type(event)){
+                case BTSTACK_EVENT_STATE:
+                    if (btstack_event_state_get_state(event) != HCI_STATE_WORKING) break;
+                    dump_supported_codecs();
+                    break;
+
                 case HCI_EVENT_SCO_CAN_SEND_NOW:
                     sco_demo_send(sco_handle);
                     break;
-
-                case HCI_EVENT_COMMAND_COMPLETE:
-                    if (HCI_EVENT_IS_COMMAND_COMPLETE(event, hci_read_local_supported_features)){
-                        dump_supported_codecs();
-                    }
-                    break;
-
+                    
                 case HCI_EVENT_HFP_META:
-                    switch (event[2]) {   
+                    switch (hci_event_hfp_meta_get_subevent_code(event)) {
                         case HFP_SUBEVENT_SERVICE_LEVEL_CONNECTION_ESTABLISHED:
-                            acl_handle = hfp_subevent_service_level_connection_established_get_con_handle(event);
+                            status = hfp_subevent_service_level_connection_established_get_status(event);
+                            if (status != ERROR_CODE_SUCCESS){
+                                printf("Connection failed, status 0x%02x\n", status);
+                                break;
+                            }
+                            acl_handle = hfp_subevent_service_level_connection_established_get_acl_handle(event);
                             hfp_subevent_service_level_connection_established_get_bd_addr(event, device_addr);
                             printf("Service level connection established %s.\n\n", bd_addr_to_str(device_addr));
                             break;
@@ -470,50 +488,63 @@ static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * even
                             printf("Service level connection released.\n\n");
                             break;
                         case HFP_SUBEVENT_AUDIO_CONNECTION_ESTABLISHED:
-                            if (hfp_subevent_audio_connection_established_get_status(event)){
-                                printf("Audio connection establishment failed with status %u\n", hfp_subevent_audio_connection_established_get_status(event));
-                            } else {
-                                sco_handle = hfp_subevent_audio_connection_established_get_handle(event);
-                                printf("Audio connection established with SCO handle 0x%04x.\n", sco_handle);
-                                negotiated_codec = hfp_subevent_audio_connection_established_get_negotiated_codec(event);
-                                switch (negotiated_codec){
-                                    case 0x01:
-                                        printf("Using CVSD codec.\n");
-                                        break;
-                                    case 0x02:
-                                        printf("Using mSBC codec.\n");
-                                        break;
-                                    default:
-                                        printf("Using unknown codec 0x%02x.\n", negotiated_codec);
-                                        break;
-                                }
-                                sco_demo_set_codec(negotiated_codec);
-                                hci_request_sco_can_send_now_event();
+                            status = hfp_subevent_audio_connection_established_get_status(event);
+                            if (status != ERROR_CODE_SUCCESS){
+                                printf("Audio connection establishment failed with status 0x%02x\n", status);
+                                break;
+                            } 
+                            sco_handle = hfp_subevent_audio_connection_established_get_sco_handle(event);
+                            printf("Audio connection established with SCO handle 0x%04x.\n", sco_handle);
+                            negotiated_codec = hfp_subevent_audio_connection_established_get_negotiated_codec(event);
+                            switch (negotiated_codec){
+                                case 0x01:
+                                    printf("Using CVSD codec.\n");
+                                    break;
+                                case 0x02:
+                                    printf("Using mSBC codec.\n");
+                                    break;
+                                default:
+                                    printf("Using unknown codec 0x%02x.\n", negotiated_codec);
+                                    break;
                             }
+                            sco_demo_set_codec(negotiated_codec);
+                            hci_request_sco_can_send_now_event();
                             break;
+
+                        case HFP_SUBEVENT_CALL_ANSWERED:
+                            printf("Call answered\n");
+                            break;
+
+                        case HFP_SUBEVENT_CALL_TERMINATED:
+                            printf("Call terminated\n");
+                            break;
+
                         case HFP_SUBEVENT_AUDIO_CONNECTION_RELEASED:
                             sco_handle = HCI_CON_HANDLE_INVALID;
                             printf("Audio connection released\n");
                             sco_demo_close();
                             break;
-                        case HFP_SUBEVENT_COMPLETE:
-                            switch (cmd){
-                                case 'd':
-                                    printf("HFP AG registration status update enabled.\n");
-                                    break;
-                                case 'e':
-                                    printf("HFP AG registration status update for individual indicators set.\n");
-                                    break;
-                                default:
-                                    break;
+                        case  HFP_SUBEVENT_COMPLETE:
+                            status = hfp_subevent_complete_get_status(event);
+                            if (status == ERROR_CODE_SUCCESS){
+                                printf("Cmd \'%c\' succeeded\n", cmd);
+                            } else {
+                                printf("Cmd \'%c\' failed with status 0x%02x\n", cmd, status);
                             }
                             break;
+
+                        case HFP_SUBEVENT_AG_INDICATOR_MAPPING:
+                            printf("AG Indicator Mapping | INDEX %d: range [%d, %d], name '%s'\n", 
+                                hfp_subevent_ag_indicator_mapping_get_indicator_index(event), 
+                                hfp_subevent_ag_indicator_mapping_get_indicator_min_range(event),
+                                hfp_subevent_ag_indicator_mapping_get_indicator_max_range(event),
+                                (const char*) hfp_subevent_ag_indicator_mapping_get_indicator_name(event));
+                            break;
+
                         case HFP_SUBEVENT_AG_INDICATOR_STATUS_CHANGED:
-                            printf("AG_INDICATOR_STATUS_CHANGED, AG indicator (index: %d) to: %d of range [%d, %d], name '%s'\n", 
+                            printf("AG Indicator Status  | INDEX %d: status %d, '%s'\n", 
                                 hfp_subevent_ag_indicator_status_changed_get_indicator_index(event), 
                                 hfp_subevent_ag_indicator_status_changed_get_indicator_status(event),
-                                hfp_subevent_ag_indicator_status_changed_get_indicator_min_range(event),
-                                hfp_subevent_ag_indicator_status_changed_get_indicator_max_range(event),
                                 (const char*) hfp_subevent_ag_indicator_status_changed_get_indicator_name(event));
                             break;
                         case HFP_SUBEVENT_NETWORK_OPERATOR_CHANGED:
@@ -523,12 +554,18 @@ static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * even
                                 (char *) hfp_subevent_network_operator_changed_get_network_operator_name(event));          
                             break;
                         case HFP_SUBEVENT_EXTENDED_AUDIO_GATEWAY_ERROR:
-                            printf("EXTENDED_AUDIO_GATEWAY_ERROR_REPORT, status : %d\n", 
+                            printf("EXTENDED_AUDIO_GATEWAY_ERROR_REPORT, status: 0x%02x\n",
                                 hfp_subevent_extended_audio_gateway_error_get_error(event));
                             break;
+                        case HFP_SUBEVENT_START_RINGING:
+                            printf("** START Ringing **\n");
+                            break;
                         case HFP_SUBEVENT_RING:
                             printf("** Ring **\n");
                             break;
+                        case HFP_SUBEVENT_STOP_RINGING:
+                            printf("** STOP Ringing **\n");
+                            break;
                         case HFP_SUBEVENT_NUMBER_FOR_VOICE_TAG:
                             printf("Phone number for voice tag: %s\n", 
                                 (const char *) hfp_subevent_number_for_voice_tag_get_number(event));
@@ -539,10 +576,11 @@ static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * even
                             break;
                         case HFP_SUBEVENT_MICROPHONE_VOLUME:
                             printf("Microphone volume: gain %u\n",
-                                hfp_subevent_microphone_volume_get_gain(event));
+                            hfp_subevent_microphone_volume_get_gain(event));
                             break;
                         case HFP_SUBEVENT_CALLING_LINE_IDENTIFICATION_NOTIFICATION:
-                            printf("Caller ID, number %s\n", hfp_subevent_calling_line_identification_notification_get_number(event));
+                            printf("Caller ID, number '%s', alpha '%s'\n", (const char *) hfp_subevent_calling_line_identification_notification_get_number(event),
+                                   (const char *) hfp_subevent_calling_line_identification_notification_get_alpha(event));
                             break;
                         case HFP_SUBEVENT_ENHANCED_CALL_STATUS:
                             printf("Enhanced call status:\n");
@@ -554,8 +592,59 @@ static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * even
                             printf("  - type      : %d \n", hfp_subevent_enhanced_call_status_get_bnip_type(event));
                             printf("  - number    : %s \n", hfp_subevent_enhanced_call_status_get_bnip_number(event));
                             break;
+                        
+                        case HFP_SUBEVENT_VOICE_RECOGNITION_ACTIVATED:
+                            status = hfp_subevent_voice_recognition_activated_get_status(event);
+                            if (status != ERROR_CODE_SUCCESS){
+                                printf("Voice Recognition Activate command failed\n");
+                                break;
+                            }
+                            
+                            switch (hfp_subevent_voice_recognition_activated_get_enhanced(event)){
+                                case 0: 
+                                    printf("\nVoice recognition ACTIVATED\n\n");
+                                    break;
+                                default:
+                                    printf("\nEnhanced voice recognition ACTIVATED.\n");
+                                    printf("Start new audio enhanced voice recognition session %s\n\n", bd_addr_to_str(device_addr));
+                                    status = hfp_hf_enhanced_voice_recognition_report_ready_for_audio(acl_handle);
+                                    break;
+                            }
+                            break;
+            
+                        case HFP_SUBEVENT_VOICE_RECOGNITION_DEACTIVATED:
+                            status = hfp_subevent_voice_recognition_deactivated_get_status(event);
+                            if (status != ERROR_CODE_SUCCESS){
+                                printf("Voice Recognition Deactivate command failed\n");
+                                break;
+                            }
+                            printf("\nVoice Recognition DEACTIVATED\n\n");
+                            break;
+
+                        case HFP_SUBEVENT_ENHANCED_VOICE_RECOGNITION_HF_READY_FOR_AUDIO:
+                            status = hfp_subevent_enhanced_voice_recognition_hf_ready_for_audio_get_status(event);
+                            report_status(status, "Enhanced Voice recognition: READY FOR AUDIO");
+                            break;
+
+                        case HFP_SUBEVENT_ENHANCED_VOICE_RECOGNITION_AG_READY_TO_ACCEPT_AUDIO_INPUT:
+                            printf("\nEnhanced Voice recognition AG status: AG READY TO ACCEPT AUDIO INPUT\n\n");                            
+                            break;
+                        case HFP_SUBEVENT_ENHANCED_VOICE_RECOGNITION_AG_IS_STARTING_SOUND:
+                            printf("\nEnhanced Voice recognition AG status: AG IS STARTING SOUND\n\n");                            
+                            break;
+                        case HFP_SUBEVENT_ENHANCED_VOICE_RECOGNITION_AG_IS_PROCESSING_AUDIO_INPUT:
+                            printf("\nEnhanced Voice recognition AG status: AG IS PROCESSING AUDIO INPUT\n\n");                            
+                            break;
+                        
+                        case HFP_SUBEVENT_ENHANCED_VOICE_RECOGNITION_AG_MESSAGE:
+                            printf("\nEnhanced Voice recognition AG message: \'%s\'\n", hfp_subevent_enhanced_voice_recognition_ag_message_get_text(event));                            
+                            break;
+
+                        case HFP_SUBEVENT_ECHO_CANCELING_AND_NOISE_REDUCTION_DEACTIVATE:
+                            status = hfp_subevent_echo_canceling_and_noise_reduction_deactivate_get_status(event);
+                            report_status(status, "Echo Canceling and Noise Reduction Deactivate");
+                            break;
                         default:
-                            printf("event not handled %u\n", event[2]);
                             break;
                     }
                     break;
@@ -595,12 +684,21 @@ int btstack_main(int argc, const char * argv[]){
     // init L2CAP
     l2cap_init();
 
+#ifdef ENABLE_BLE
+    // Initialize LE Security Manager. Needed for cross-transport key derivation
+    sm_init();
+#endif
+
     uint16_t hf_supported_features          =
         (1<<HFP_HFSF_ESCO_S4)               |
         (1<<HFP_HFSF_CLI_PRESENTATION_CAPABILITY) |
         (1<<HFP_HFSF_HF_INDICATORS)         |
         (1<<HFP_HFSF_CODEC_NEGOTIATION)     |
         (1<<HFP_HFSF_ENHANCED_CALL_STATUS)  |
+        (1<<HFP_HFSF_VOICE_RECOGNITION_FUNCTION)  |
+        (1<<HFP_HFSF_ENHANCED_VOICE_RECOGNITION_STATUS) |
+        (1<<HFP_HFSF_VOICE_RECOGNITION_TEXT) |
+        (1<<HFP_HFSF_EC_NR_FUNCTION) |
         (1<<HFP_HFSF_REMOTE_VOLUME_CONTROL);
     int wide_band_speech = 1;
 
@@ -622,6 +720,15 @@ int btstack_main(int argc, const char * argv[]){
     hci_register_sco_packet_handler(&packet_handler);
     hci_register_sco_packet_handler(&packet_handler);
 
+    // Service Class: Audio, Major Device Class: Audio, Minor: Hands-Free device
+    gap_set_class_of_device(0x200408);
+
+    // allow for role switch in general and sniff mode
+    gap_set_default_link_policy_settings( LM_LINK_POLICY_ENABLE_ROLE_SWITCH | LM_LINK_POLICY_ENABLE_SNIFF_MODE );
+
+    // allow for role switch on outgoing connections - this allows HFP AG, e.g. smartphone, to become master when we re-connect to it
+    gap_set_allow_role_switch(true);
+
     // register for HFP events
     hfp_hf_register_packet_handler(packet_handler);
 
