@@ -20,8 +20,8 @@
  * THIS SOFTWARE IS PROVIDED BY BLUEKITCHEN GMBH AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATTHIAS
- * RINGWALD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL BLUEKITCHEN
+ * GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
@@ -42,16 +42,17 @@
  * Author: Ole Reinhardt <ole.reinhardt@kernelconcepts.de>
  */
 
-/* EXAMPLE_START(panu_demo): PANU Demo
+/* EXAMPLE_START(panu_demo): BNEP/PANU (Linux only)
  *
  * @text This example implements both a PANU client and a server. In server mode, it 
  * sets up a BNEP server and registers a PANU SDP record and waits for incoming connections.
  * In client mode, it connects to a remote device, does an SDP Query to identify the PANU
  * service and initiates a BNEP connection.
  *
- * Note: currently supported only on Linux and Mac.
+ * Note: currently supported only on Linux and provides a TAP network interface which you
+ *       can configure yourself.
  *
- * To enable client mode, uncomment ENABLE_PANU_CLIENT below
+ * To enable client mode, uncomment ENABLE_PANU_CLIENT below.
  */
 
 #include <stdio.h>
@@ -66,18 +67,23 @@
 #define NETWORK_TYPE_ARP        0x0806
 #define NETWORK_TYPE_IPv6       0x86DD
 
-static int record_id = -1;
-static uint16_t bnep_l2cap_psm      = 0;
-static uint32_t bnep_remote_uuid    = 0;
-static uint16_t bnep_version        = 0;
 static uint16_t bnep_cid            = 0;
 
+#ifdef ENABLE_PANU_CLIENT
+
+static int record_id = -1;
+
+static uint16_t bnep_version        = 0;
+static uint32_t bnep_remote_uuid    = 0;
+static uint16_t bnep_l2cap_psm      = 0;
+
 static uint16_t sdp_bnep_l2cap_psm      = 0;
 static uint16_t sdp_bnep_version        = 0;
 static uint32_t sdp_bnep_remote_uuid    = 0;
 
 static uint8_t   attribute_value[1000];
 static const unsigned int attribute_value_buffer_size = sizeof(attribute_value);
+#endif
 
 // MBP 2016
 static const char * remote_addr_string = "78:4F:43:8C:B2:5D";
@@ -106,9 +112,13 @@ static void network_send_packet_callback(const uint8_t * packet, uint16_t size);
 
 static void panu_setup(void){
 
-    // Initialize L2CAP 
+    // Initialize L2CAP
     l2cap_init();
 
+#ifdef ENABLE_BLE
+    // Initialize LE Security Manager. Needed for cross-transport key derivation
+    sm_init();
+#endif
 
     // init SDP, create record for PANU and register with SDP
     sdp_init();
