@@ -20,8 +20,8 @@
  * THIS SOFTWARE IS PROVIDED BY BLUEKITCHEN GMBH AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATTHIAS
- * RINGWALD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL BLUEKITCHEN
+ * GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
@@ -53,6 +53,11 @@
 #include "classic/hfp.h"
 #include "classic/hfp_msbc.h"
 
+#ifdef _MSC_VER
+// ignore deprecated warning for fopen
+#pragma warning(disable : 4996)
+#endif
+
 #ifdef HAVE_POSIX_FILE_IO
 #include "wav_util.h"
 #endif
@@ -71,7 +76,6 @@
 // number of sco packets until 'report' on console
 #define SCO_REPORT_PERIOD           100
 
-// #define ENABLE_SCO_STEREO_PLAYBACK
 
 #ifdef HAVE_POSIX_FILE_IO
 // length and name of wav file on disk
@@ -205,11 +209,7 @@ static void playback_callback(int16_t * buffer, uint16_t num_samples){
     // fill with silence while paused
     if (audio_output_paused){
         if (btstack_ring_buffer_bytes_available(&audio_output_ring_buffer) < prebuffer_bytes){
-#ifdef ENABLE_SCO_STEREO_PLAYBACK
-            memset(buffer, 0, num_samples * BYTES_PER_FRAME * 2);
-#else
             memset(buffer, 0, num_samples * BYTES_PER_FRAME);
-#endif 
            return;
         } else {
             // resume playback
@@ -219,32 +219,13 @@ static void playback_callback(int16_t * buffer, uint16_t num_samples){
 
     // get data from ringbuffer
     uint32_t bytes_read = 0;
-#ifdef ENABLE_SCO_STEREO_PLAYBACK
-    while (num_samples){
-        int16_t temp[16];
-        unsigned int bytes_to_read = btstack_min(num_samples * BYTES_PER_FRAME, sizeof(temp));
-        btstack_ring_buffer_read(&audio_output_ring_buffer, (uint8_t *) &temp[0], bytes_to_read, &bytes_read);
-        if (bytes_read == 0) break;
-        unsigned int i;
-        for (i=0;i<bytes_read / BYTES_PER_FRAME;i++){
-            *buffer++ = temp[i];
-            *buffer++ = temp[i];
-            num_samples--;
-        }
-    }
-#else
     btstack_ring_buffer_read(&audio_output_ring_buffer, (uint8_t *) buffer, num_samples * BYTES_PER_FRAME, &bytes_read);
     num_samples -= bytes_read / BYTES_PER_FRAME;
     buffer      += bytes_read / BYTES_PER_FRAME;
-#endif
 
     // fill with 0 if not enough
     if (num_samples){
-#ifdef ENABLE_SCO_STEREO_PLAYBACK
-        memset(buffer, 0, num_samples * BYTES_PER_FRAME * 2);
-#else
         memset(buffer, 0, num_samples * BYTES_PER_FRAME);
-#endif
         audio_output_paused = 1;
     }
 }
@@ -268,11 +249,7 @@ static int audio_initialize(int sample_rate){
     const btstack_audio_sink_t * audio_sink = btstack_audio_sink_get_instance();
     if (!audio_sink) return 0;
 
-#ifdef ENABLE_SCO_STEREO_PLAYBACK
-    audio_sink->init(2, sample_rate, &playback_callback);
-#else
     audio_sink->init(1, sample_rate, &playback_callback);
-#endif
     audio_sink->start_stream();
 
     audio_output_paused  = 1;
@@ -415,7 +392,10 @@ static void sco_demo_receive_CVSD(uint8_t * packet, uint16_t size){
         audio_frame_in[i] = little_endian_read_16(packet, 3 + i * 2);
     }
 
-    btstack_cvsd_plc_process_data(&cvsd_plc_state, audio_frame_in, num_samples, audio_frame_out);
+    // treat packet as bad frame if controller does not report 'all good'
+    bool bad_frame = (packet[1] & 0x30) != 0;
+
+    btstack_cvsd_plc_process_data(&cvsd_plc_state, bad_frame, audio_frame_in, num_samples, audio_frame_out);
 
 #ifdef SCO_WAV_FILENAME
     // Samples in CVSD SCO packet are in little endian, ready for wav files (take shortcut)
@@ -475,6 +455,12 @@ void sco_demo_set_codec(uint8_t codec){
 }
 
 void sco_demo_init(void){
+
+#ifdef ENABLE_CLASSIC_LEGACY_CONNECTIONS_FOR_SCO_DEMOS
+    printf("Disable BR/EDR Secure Connctions due to incompatibilities with SCO connections\n");
+    gap_secure_connections_enable(false);
+#endif
+
 	// status
 #if SCO_DEMO_MODE == SCO_DEMO_MODE_MICROPHONE
     printf("SCO Demo: Sending and receiving audio via btstack_audio.\n");
