@@ -20,8 +20,8 @@
  * THIS SOFTWARE IS PROVIDED BY BLUEKITCHEN GMBH AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATTHIAS
- * RINGWALD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL BLUEKITCHEN
+ * GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
@@ -53,7 +53,6 @@
 // SCO Data     0 0 0x83 Isochronous (IN)
 // SCO Data     0 0 0x03 Isochronous (Out)
 
-#include <stdio.h>
 #include <strings.h>
 #include <string.h>
 #include <unistd.h>   /* UNIX standard function definitions */
@@ -66,6 +65,7 @@
 #include "btstack_debug.h"
 #include "hci.h"
 #include "hci_transport.h"
+#include "hci_transport_usb.h"
 
 // deal with changes in libusb API:
 #ifdef LIBUSB_API_VERSION
@@ -165,6 +165,15 @@ static struct libusb_transfer *acl_out_transfer;
 static struct libusb_transfer *event_in_transfer[EVENT_IN_BUFFER_COUNT];
 static struct libusb_transfer *acl_in_transfer[ACL_IN_BUFFER_COUNT];
 
+// known devices
+typedef struct {
+    btstack_linked_item_t next;
+    uint16_t vendor_id;
+    uint16_t product_id;
+} usb_known_device_t;
+
+static btstack_linked_list_t usb_knwon_devices;
+
 #ifdef ENABLE_SCO_OVER_HCI
 
 #ifdef _WIN32
@@ -223,13 +232,29 @@ static int acl_out_addr;
 static int sco_in_addr;
 static int sco_out_addr;
 
-// device path
+// device info
 static int usb_path_len;
 static uint8_t usb_path[USB_MAX_PATH_LEN];
+static uint16_t usb_vendor_id;
+static uint16_t usb_product_id;
 
 // transport interface state
 static int usb_transport_open;
 
+static void hci_transport_h2_libusb_emit_usb_info(void) {
+    uint8_t event[7 + USB_MAX_PATH_LEN];
+    uint16_t pos = 0;
+    event[pos++] = HCI_EVENT_TRANSPORT_USB_INFO;
+    event[pos++] = 5 + usb_path_len;
+    little_endian_store_16(event, pos, usb_vendor_id);
+    pos+=2;
+    little_endian_store_16(event, pos, usb_product_id);
+    pos+=2;
+    event[pos++] = usb_path_len;
+    memcpy(&event[pos], usb_path, usb_path_len);
+    pos += usb_path_len;
+    (*packet_handler)(HCI_EVENT_PACKET, event, pos);
+}
 
 #ifdef ENABLE_SCO_OVER_HCI
 static void sco_ring_init(void){
@@ -241,6 +266,15 @@ static int sco_ring_have_space(void){
 }
 #endif
 
+void hci_transport_usb_add_device(uint16_t vendor_id, uint16_t product_id) {
+    usb_known_device_t * device = malloc(sizeof(usb_known_device_t));
+    if (device != NULL) {
+        device->vendor_id = vendor_id;
+        device->product_id = product_id;
+        btstack_linked_list_add(&usb_knwon_devices, (btstack_linked_item_t *) device);
+    }
+}
+
 void hci_transport_usb_set_path(int len, uint8_t * port_numbers){
     if (len > USB_MAX_PATH_LEN || !port_numbers){
         log_error("hci_transport_usb_set_path: len or port numbers invalid");
@@ -286,7 +320,7 @@ LIBUSB_CALL static void async_callback(struct libusb_transfer *transfer){
         for (c=0;c<SCO_IN_BUFFER_COUNT;c++){
             if (transfer == sco_in_transfer[c]){
                 libusb_free_transfer(transfer);
-                sco_in_transfer[c] = 0;
+                sco_in_transfer[c] = NULL;
                 return;
             }
         }
@@ -295,7 +329,7 @@ LIBUSB_CALL static void async_callback(struct libusb_transfer *transfer){
             if (transfer == sco_out_transfers[c]){
                 sco_out_transfers_in_flight[c] = 0;
                 libusb_free_transfer(transfer);
-                sco_out_transfers[c] = 0;
+                sco_out_transfers[c] = NULL;
                 return;
             }
         }
@@ -434,6 +468,9 @@ static void handle_isochronous_data(uint8_t * buffer, uint16_t size){
                 packet_handler(HCI_SCO_DATA_PACKET, sco_buffer, sco_read_pos);
                 sco_state_machine_init();
                 break;
+			default:
+				btstack_assert(false);
+				break;
         }
     }
 }
@@ -471,8 +508,6 @@ static void handle_completed_transfer(struct libusb_transfer *transfer){
             }
             if (!pack->actual_length) continue;
             uint8_t * data = libusb_get_iso_packet_buffer_simple(transfer, i);
-            // printf_hexdump(data, pack->actual_length);
-            // log_info("handle_isochronous_data,size %u/%u", pack->length, pack->actual_length);
             handle_isochronous_data(data, pack->actual_length);
         }
         resubmit = 1;
@@ -575,31 +610,6 @@ static void usb_process_ts(btstack_timer_source_t *timer) {
     return;
 }
 
-#ifndef HAVE_USB_VENDOR_ID_AND_PRODUCT_ID
-
-// list of known devices, using VendorID/ProductID tuples
-static const uint16_t known_bt_devices[] = {
-    // DeLOCK Bluetooth 4.0
-    0x0a5c, 0x21e8,
-    // Asus BT400
-    0x0b05, 0x17cb,
-    // BCM20702B0 (Generic USB Detuned Class 1 @ 20 MHz)
-    0x0a5c, 0x22be,
-    // Zephyr e.g nRF52840-PCA10056
-    0x2fe3, 0x0100,
-};
-
-static int num_known_devices = sizeof(known_bt_devices) / sizeof(uint16_t) / 2;
-
-static int is_known_bt_device(uint16_t vendor_id, uint16_t product_id){
-    int i;
-    for (i=0; i<num_known_devices; i++){
-        if (known_bt_devices[i*2] == vendor_id && known_bt_devices[i*2+1] == product_id){
-            return 1;
-        }
-    }
-    return 0;
-}
 
 static int scan_for_bt_endpoints(libusb_device *dev) {
     int r;
@@ -666,6 +676,41 @@ static int scan_for_bt_endpoints(libusb_device *dev) {
     return 0;
 }
 
+#ifndef HAVE_USB_VENDOR_ID_AND_PRODUCT_ID
+
+// list of known devices, using VendorID/ProductID tuples
+static const uint16_t known_bluetooth_devices[] = {
+    // BCM20702A0 - DeLOCK Bluetooth 4.0
+    0x0a5c, 0x21e8,
+    // BCM20702A0 - Asus BT400
+    0x0b05, 0x17cb,
+    // BCM20702B0 - Generic USB Detuned Class 1 @ 20 MHz
+    0x0a5c, 0x22be,
+    // nRF5x Zephyr USB HCI, e.g nRF52840-PCA10056
+    0x2fe3, 0x0100,
+    0x2fe3, 0x000b,
+};
+
+static int num_known_devices = sizeof(known_bluetooth_devices) / sizeof(uint16_t) / 2;
+
+static int is_known_bt_device(uint16_t vendor_id, uint16_t product_id){
+    int i;
+    for (i=0; i<num_known_devices; i++){
+        if (known_bluetooth_devices[i*2] == vendor_id && known_bluetooth_devices[i*2+1] == product_id){
+            return 1;
+        }
+    }
+    btstack_linked_list_iterator_t it;
+    btstack_linked_list_iterator_init(&it, &usb_knwon_devices);
+    while (btstack_linked_list_iterator_has_next(&it)) {
+        usb_known_device_t * device = (usb_known_device_t *) btstack_linked_list_iterator_next(&it);
+        if (device->vendor_id != vendor_id) continue;
+        if (device->product_id != product_id) continue;
+        return 1;
+    }
+    return 0;
+}
+
 // returns index of found device or -1
 static int scan_for_bt_device(libusb_device **devs, int start_index) {
     int i;
@@ -686,7 +731,6 @@ static int scan_for_bt_device(libusb_device **devs, int start_index) {
         // The class code (bDeviceClass) is 0xE0 – Wireless Controller. 
         // The SubClass code (bDeviceSubClass) is 0x01 – RF Controller. 
         // The Protocol code (bDeviceProtocol) is 0x01 – Bluetooth programming.
-        // if (desc.bDeviceClass == 0xe0 && desc.bDeviceSubClass == 0x01 && desc.bDeviceProtocol == 0x01){
         if (desc.bDeviceClass == 0xE0 && desc.bDeviceSubClass == 0x01 && desc.bDeviceProtocol == 0x01) {
             return i;
         }
@@ -702,23 +746,15 @@ static int scan_for_bt_device(libusb_device **devs, int start_index) {
 
 static int prepare_device(libusb_device_handle * aHandle){
 
-    // print device path
-    uint8_t port_numbers[USB_MAX_PATH_LEN];
+    // get device path
     libusb_device * device = libusb_get_device(aHandle);
-    int path_len = libusb_get_port_numbers(device, port_numbers, USB_MAX_PATH_LEN);
-    printf("USB Path: ");
-    int i;
-    for (i=0;i<path_len;i++){
-        if (i) printf("-");
-        printf("%02x", port_numbers[i]);
-    }
-    printf("\n");
+    usb_path_len = libusb_get_port_numbers(device, usb_path, USB_MAX_PATH_LEN);
 
     int r;
     int kernel_driver_detached = 0;
 
-    // Detach OS driver (not possible for OS X, FreeBSD, and WIN32)
-#if !defined(__APPLE__) && !defined(_WIN32) && !defined(__FreeBSD__)
+    // Detach OS driver (not possible for OS X, FreeBSD, and Windows)
+#if !defined(__APPLE__) && !defined(_WIN32) && !defined(__CYGWIN__) && !defined(__FreeBSD__)
     r = libusb_kernel_driver_active(aHandle, 0);
     if (r < 0) {
         log_error("libusb_kernel_driver_active error %d", r);
@@ -763,12 +799,21 @@ static int prepare_device(libusb_device_handle * aHandle){
     }
 
 #ifdef ENABLE_SCO_OVER_HCI
-    log_info("claiming interface 1...");
-    r = libusb_claim_interface(aHandle, 1);
-    if (r < 0) {
-        log_error("Error %d claiming interface 1: - disabling SCO over HCI", r);
-    } else {
-        sco_enabled = 1;
+    // get endpoints from interface descriptor
+    struct libusb_config_descriptor *config_descriptor;
+    r = libusb_get_active_config_descriptor(device, &config_descriptor);
+    if (r >= 0){
+        int num_interfaces = config_descriptor->bNumInterfaces;
+        if (num_interfaces > 1) {
+            r = libusb_claim_interface(aHandle, 1);
+            if (r < 0) {
+                log_error("Error %d claiming interface 1: - disabling SCO over HCI", r);
+            } else {
+                sco_enabled = 1;
+            }
+        } else {
+            log_info("Device has only on interface, disabling SCO over HCI");
+        }
     }
 #endif
 
@@ -778,6 +823,14 @@ static int prepare_device(libusb_device_handle * aHandle){
 static libusb_device_handle * try_open_device(libusb_device * device){
     int r;
 
+    r = libusb_get_device_descriptor(device, &desc);
+    if (r < 0) {
+        log_error("libusb_get_device_descriptor failed!");
+        return NULL;
+    }
+    usb_vendor_id = desc.idVendor;
+    usb_product_id = desc.idProduct;
+
     libusb_device_handle * dev_handle;
     r = libusb_open(device, &dev_handle);
 
@@ -805,7 +858,6 @@ static libusb_device_handle * try_open_device(libusb_device * device){
 
 static int usb_sco_start(void){
 
-    printf("usb_sco_start\n");
     log_info("usb_sco_start");
 
     sco_state_machine_init();
@@ -859,24 +911,64 @@ static int usb_sco_start(void){
 
 static void usb_sco_stop(void){
 
-    printf("usb_sco_stop\n");
-
     log_info("usb_sco_stop");
     sco_shutdown = 1;
 
+    // Free SCO transfers already in queue
+    struct libusb_transfer* transfer = handle_packet;
+    struct libusb_transfer* prev_transfer = NULL;
+    while (transfer != NULL) {
+        uint16_t c;
+        bool drop_transfer = false;
+        for (c=0;c<SCO_IN_BUFFER_COUNT;c++){
+            if (transfer == sco_in_transfer[c]){
+                sco_in_transfer[c] = NULL;
+                drop_transfer = true;
+                break;
+            }
+        }
+        for (c=0;c<SCO_OUT_BUFFER_COUNT;c++){
+            if (transfer == sco_out_transfers[c]){
+                sco_out_transfers_in_flight[c] = 0;
+                sco_out_transfers[c] = NULL;
+                drop_transfer = true;
+                break;
+            }
+        }
+        struct libusb_transfer * next_transfer = (struct libusb_transfer *) transfer->user_data;
+        if (drop_transfer) {
+            log_info("Drop completed SCO transfer %p", transfer);
+            if (prev_transfer == NULL) {
+                // first item
+                handle_packet = (struct libusb_transfer *) next_transfer;
+            } else {
+                // other item
+                prev_transfer->user_data = (struct libusb_transfer *) next_transfer;
+            }
+            libusb_free_transfer(transfer);
+        } else {
+            prev_transfer = transfer;
+        }
+        transfer = next_transfer;
+    }
+
     libusb_set_debug(NULL, LIBUSB_LOG_LEVEL_ERROR);
 
     int c;
     for (c = 0 ; c < SCO_IN_BUFFER_COUNT ; c++) {
-        libusb_cancel_transfer(sco_in_transfer[c]);
+        if (sco_in_transfer[c] != NULL) {
+            libusb_cancel_transfer(sco_in_transfer[c]);
+        }
     }
 
     for (c = 0; c < SCO_OUT_BUFFER_COUNT ; c++){
         if (sco_out_transfers_in_flight[c]) {
             libusb_cancel_transfer(sco_out_transfers[c]);
         } else {
-            libusb_free_transfer(sco_out_transfers[c]);
-            sco_out_transfers[c] = 0;
+            if (sco_out_transfers[c] != NULL) {
+                libusb_free_transfer(sco_out_transfers[c]);
+                sco_out_transfers[c] = 0;
+            }
         }
     }
 
@@ -891,7 +983,7 @@ static void usb_sco_stop(void){
 
         // Cancel all synchronous transfer
         for (c = 0 ; c < SCO_IN_BUFFER_COUNT ; c++) {
-            if (sco_in_transfer[c]){
+            if (sco_in_transfer[c] != NULL){
                 completed = 0;
                 break;
             }
@@ -900,7 +992,7 @@ static void usb_sco_stop(void){
         if (!completed) continue;
 
         for (c=0; c < SCO_OUT_BUFFER_COUNT ; c++){
-            if (sco_out_transfers[c]){
+            if (sco_out_transfers[c] != NULL){
                 completed = 0;
                 break;
             }
@@ -916,7 +1008,7 @@ static void usb_sco_stop(void){
         return;
     }
 
-    printf("usb_sco_stop done\n");
+    log_info("usb_sco_stop done");
 }
 
 
@@ -967,13 +1059,16 @@ static int usb_open(void){
         return -1;
     }
 
-    dev = libusb_get_device(aHandle);
+    dev = libusb_get_device(handle);
     r = scan_for_bt_endpoints(dev);
     if (r < 0){
         usb_close();
         return -1;
     }
 
+    usb_vendor_id = USB_VENDOR_ID;
+    usb_product_id = USB_PRODUCT_ID;
+
 #else
     // Scan system for an appropriate devices
     libusb_device **devs;
@@ -1016,7 +1111,6 @@ static int usb_open(void){
         }
         if (!handle){
             log_error("USB device with given path not found");
-            printf("USB device with given path not found\n");
             return -1;
         }
     } else {
@@ -1148,6 +1242,8 @@ static int usb_open(void){
 
     usb_transport_open = 1;
 
+    hci_transport_h2_libusb_emit_usb_info();
+
     return 0;
 }
 
@@ -1183,6 +1279,8 @@ static int usb_close(void){
                 doing_pollfds = 0;
             }
 
+            /* fall through */
+
         case LIB_USB_INTERFACE_CLAIMED:
             // Cancel all transfers, ignore warnings for this
             libusb_set_debug(NULL, LIBUSB_LOG_LEVEL_ERROR);
@@ -1210,8 +1308,10 @@ static int usb_close(void){
                     log_info("cancel sco_out_transfers[%u] = %p", c, sco_out_transfers[c]);
                     libusb_cancel_transfer(sco_out_transfers[c]);
                 } else {
-                    libusb_free_transfer(sco_out_transfers[c]);
-                    sco_out_transfers[c] = 0;
+                    if (sco_out_transfers[c] != NULL){
+                        libusb_free_transfer(sco_out_transfers[c]);
+                        sco_out_transfers[c] = 0;
+                    }
                 }
             }
 #endif
@@ -1264,7 +1364,7 @@ static int usb_close(void){
                 if (!completed) continue;
 
                 for (c=0; c < SCO_OUT_BUFFER_COUNT ; c++){
-                    if (sco_out_transfers[c]){
+                    if (sco_out_transfers[c] != NULL){
                         log_info("sco_out_transfers[%u] still active (%p)", c, sco_out_transfers[c]);
                         completed = 0;
                         break;
@@ -1281,11 +1381,20 @@ static int usb_close(void){
 #endif
             log_info("Libusb shutdown complete");
 
+			/* fall through */
+
         case LIB_USB_DEVICE_OPENDED:
             libusb_close(handle);
 
+			/* fall through */
+
         case LIB_USB_OPENED:
             libusb_exit(NULL);
+            break;
+
+		default:
+			btstack_assert(false);
+			break;
     }
 
     libusb_state = LIB_USB_CLOSED;
@@ -1314,7 +1423,7 @@ static int usb_send_cmd_packet(uint8_t *packet, int size){
 
     // submit transfer
     r = libusb_submit_transfer(command_out_transfer);
-    
+
     if (r < 0) {
         usb_command_active = 0;
         log_error("Error submitting cmd transfer %d", r);
@@ -1330,7 +1439,7 @@ static int usb_send_acl_packet(uint8_t *packet, int size){
     if (libusb_state != LIB_USB_TRANSFERS_ALLOCATED) return -1;
 
     // log_info("usb_send_acl_packet enter, size %u", size);
-    
+
     // prepare transfer
     int completed = 0;
     libusb_fill_bulk_transfer(acl_out_transfer, handle, acl_out_addr, packet, size,
