@@ -1,28 +1,30 @@
 //
-// btstack_config.h for EM9304 DVK
+// btstack_config.h for Apollo 2 + EM9304 port
+//
+// Documentation: https://bluekitchen-gmbh.com/btstack/#how_to/
 //
 
-#ifndef __BTSTACK_CONFIG
-#define __BTSTACK_CONFIG
+#ifndef BTSTACK_CONFIG_H
+#define BTSTACK_CONFIG_H
 
 // Port related features
 #define HAVE_EMBEDDED_TIME_MS
 
 // BTstack features that can be enabled
 #define ENABLE_BLE
-#define ENABLE_LE_PERIPHERAL
 #define ENABLE_LE_CENTRAL
-#define ENABLE_LE_DATA_CHANNELS
+#define ENABLE_L2CAP_LE_CREDIT_BASED_FLOW_CONTROL_MODE
 #define ENABLE_LE_DATA_LENGTH_EXTENSION
-// #define ENABLE_LOG_INFO
+#define ENABLE_LE_PERIPHERAL
 #define ENABLE_LOG_ERROR
+#define ENABLE_PRINTF_HEXDUMP
 
 // BTstack configuration. buffers, sizes, ...
 #define HCI_ACL_PAYLOAD_SIZE 200
-#define MAX_NR_WHITELIST_ENTRIES 1
+#define MAX_NR_GATT_CLIENTS 1
 #define MAX_NR_HCI_CONNECTIONS 1
 #define MAX_NR_SM_LOOKUP_ENTRIES 3
-#define MAX_NR_GATT_CLIENTS 1
+#define MAX_NR_WHITELIST_ENTRIES 1
 
 #define MAX_NR_LE_DEVICE_DB_ENTRIES 1
 
