@@ -1,28 +1,33 @@
 //
-// btstack_config.h for generic POSIX H4 port
+// btstack_config.h for Raspberry Pi port
+//
+// Documentation: https://bluekitchen-gmbh.com/btstack/#how_to/
 //
 
-#ifndef __BTSTACK_CONFIG
-#define __BTSTACK_CONFIG
+#ifndef BTSTACK_CONFIG_H
+#define BTSTACK_CONFIG_H
 
 // Port related features
+#define HAVE_ASSERT
+#define HAVE_BTSTACK_STDIN
 #define HAVE_MALLOC
 #define HAVE_POSIX_FILE_IO
-#define HAVE_BTSTACK_STDIN
 
 // BTstack features that can be enabled
 #define ENABLE_BLE
 #define ENABLE_CLASSIC
+#define ENABLE_H5
 #define ENABLE_HFP_WIDE_BAND_SPEECH
 #define ENABLE_L2CAP_ENHANCED_RETRANSMISSION_MODE
 #define ENABLE_LE_CENTRAL
+#define ENABLE_L2CAP_LE_CREDIT_BASED_FLOW_CONTROL_MODE
+#define ENABLE_LE_DATA_LENGTH_EXTENSION
 #define ENABLE_LE_PERIPHERAL
 #define ENABLE_LE_SECURE_CONNECTIONS
-#define ENABLE_MICRO_ECC_FOR_LE_SECURE_CONNECTIONS
-#define ENABLE_LE_DATA_CHANNELS
-#define ENABLE_LE_DATA_LENGTH_EXTENSION
 #define ENABLE_LOG_ERROR
-#define ENABLE_LOG_INFO 
+#define ENABLE_LOG_INFO
+#define ENABLE_MICRO_ECC_FOR_LE_SECURE_CONNECTIONS
+#define ENABLE_PRINTF_HEXDUMP
 #define ENABLE_SCO_OVER_HCI
 #define ENABLE_SDP_DES_DUMP
 #define ENABLE_SOFTWARE_AES128
@@ -31,10 +36,12 @@
 // #define ENABLE_CONTROLLER_WARM_BOOT
 
 // BTstack configuration. buffers, sizes, ...
-#define HCI_INCOMING_PRE_BUFFER_SIZE 14 // sizeof benep heade, avoid memcpy
 #define HCI_ACL_PAYLOAD_SIZE (1691 + 4)
+#define HCI_INCOMING_PRE_BUFFER_SIZE 14 // sizeof BNEP header, avoid memcpy
+#define HCI_OUTGOING_PRE_BUFFER_SIZE  4
 
-#define NVM_NUM_DEVICE_DB_ENTRIES      20
+#define NVM_NUM_DEVICE_DB_ENTRIES      16
+#define NVM_NUM_LINK_KEYS              16
 
 // Mesh Configuration
 #define ENABLE_MESH
@@ -42,12 +49,12 @@
 #define ENABLE_MESH_GATT_BEARER
 #define ENABLE_MESH_PB_ADV
 #define ENABLE_MESH_PB_GATT
-#define ENABLE_MESH_PROXY_SERVER
 #define ENABLE_MESH_PROVISIONER
+#define ENABLE_MESH_PROXY_SERVER
 
+#define MAX_NR_MESH_SUBNETS            2
 #define MAX_NR_MESH_TRANSPORT_KEYS    16
 #define MAX_NR_MESH_VIRTUAL_ADDRESSES 16
-#define MAX_NR_MESH_SUBNETS            2
 
 // allow for one NetKey update
 #define MAX_NR_MESH_NETWORK_KEYS      (MAX_NR_MESH_SUBNETS+1)
