@@ -1,21 +1,24 @@
-// 
+//
 // btstack_config.h for SAMV71 + ATWILC3000
 //
+// Documentation: https://bluekitchen-gmbh.com/btstack/#how_to/
+//
 
-#ifndef __BTSTACK_CONFIG
-#define __BTSTACK_CONFIG
+#ifndef BTSTACK_CONFIG_H
+#define BTSTACK_CONFIG_H
 
-#define HAVE_INIT_SCRIPT
-#define HAVE_EMBEDDED_TICK
-#define HAVE_UART_DMA_SET_FLOWCONTROL
 #define HAVE_BTSTACK_STDIN
+#define HAVE_EMBEDDED_TICK
+#define HAVE_INIT_SCRIPT
+#define HAVE_UART_DMA_SET_FLOWCONTROL
 
 #define ENABLE_BLE
-#define ENABLE_LOG_INFO
-#define ENABLE_LOG_ERROR
-#define ENABLE_LE_PERIPHERAL
 #define ENABLE_LE_CENTRAL
-#define ENABLE_LE_DATA_CHANNELS
+#define ENABLE_L2CAP_LE_CREDIT_BASED_FLOW_CONTROL_MODE
+#define ENABLE_LE_PERIPHERAL
+#define ENABLE_LOG_ERROR
+#define ENABLE_LOG_INFO
+#define ENABLE_PRINTF_HEXDUMP
 
 #define HCI_ACL_PAYLOAD_SIZE 300
 
@@ -32,17 +35,5 @@
 #define MAX_NR_SM_LOOKUP_ENTRIES 1
 #define MAX_NR_WHITELIST_ENTRIES 1
 
-// no classic services
-#define MAX_NR_AVDTP_CONNECTIONS 0
-#define MAX_NR_AVDTP_STREAM_ENDPOINTS 0
-#define MAX_NR_AVRCP_CONNECTIONS 0
-#define MAX_NR_BNEP_CHANNELS 0
-#define MAX_NR_BNEP_SERVICES 0
-#define MAX_NR_DB_MEM_DEVICE_LINK_KEYS  0
-#define MAX_NR_BTSTACK_LINK_KEY_DB_MEMORY_ENTRIES  0
-#define MAX_NR_HFP_CONNECTIONS 0
-#define MAX_NR_L2CAP_CHANNELS  0
-#define MAX_NR_L2CAP_SERVICES  0
-
 #endif
 
