@@ -1,40 +1,34 @@
 //
-// btstack_config.h for EM9304 DVK
+// btstack_config.h for STM32 L073RZ Nucleo + EM9304 Development Kit port
+//
+// Documentation: https://bluekitchen-gmbh.com/btstack/#how_to/
 //
 
-#ifndef __BTSTACK_CONFIG
-#define __BTSTACK_CONFIG
+#ifndef BTSTACK_CONFIG_H
+#define BTSTACK_CONFIG_H
 
 // Port related features
-#define HAVE_EMBEDDED_TIME_MS
 #define HAVE_EM9304_PATCH_CONTAINER
+#define HAVE_EMBEDDED_TIME_MS
 
 // BTstack features that can be enabled
 #define ENABLE_BLE
-#define ENABLE_LE_PERIPHERAL
 #define ENABLE_LE_CENTRAL
 #define ENABLE_LE_DATA_LENGTH_EXTENSION
-#define ENABLE_LOG_INFO
+#define ENABLE_LE_PERIPHERAL
 #define ENABLE_LOG_ERROR
+#define ENABLE_LOG_INFO
+#define ENABLE_PRINTF_HEXDUMP
 #define ENABLE_SEGGER_RTT
 
 // BTstack configuration. buffers, sizes, ...
 #define HCI_ACL_PAYLOAD_SIZE 100
-#define MAX_NR_WHITELIST_ENTRIES 1
-#define MAX_NR_HCI_CONNECTIONS 1
-#define MAX_NR_SM_LOOKUP_ENTRIES 3
-#define MAX_NR_L2CAP_SERVICES 1
-#define MAX_NR_L2CAP_CHANNELS 1
 #define MAX_NR_GATT_CLIENTS 1
-
-#define MAX_NR_RFCOMM_MULTIPLEXERS 0
-#define MAX_NR_RFCOMM_SERVICES 0
-#define MAX_NR_RFCOMM_CHANNELS 0
-#define MAX_NR_HFP_CONNECTIONS 0
-#define MAX_NR_BTSTACK_LINK_KEY_DB_MEMORY_ENTRIES 0
-#define MAX_NR_BNEP_SERVICES 0
-#define MAX_NR_BNEP_CHANNELS 0
-#define MAX_NR_SERVICE_RECORD_ITEMS 1
+#define MAX_NR_HCI_CONNECTIONS 1
+#define MAX_NR_L2CAP_CHANNELS 1
+#define MAX_NR_L2CAP_SERVICES 1
+#define MAX_NR_SM_LOOKUP_ENTRIES 3
+#define MAX_NR_WHITELIST_ENTRIES 1
 
 // LE Device DB using TLV on top of Flash Sector interface
 #define NVM_NUM_DEVICE_DB_ENTRIES 16
