@@ -20,8 +20,8 @@
  * THIS SOFTWARE IS PROVIDED BY BLUEKITCHEN GMBH AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATTHIAS
- * RINGWALD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL BLUEKITCHEN
+ * GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
@@ -45,9 +45,9 @@
  *  Created by Matthias Ringwald on 4/29/09.
  */
 
-#include <inttypes.h>
 
 #include "btstack_config.h"
+#include "hci_transport_h4.h"
 
 #include "btstack_debug.h"
 #include "hci.h"
@@ -55,6 +55,8 @@
 #include "bluetooth_company_id.h"
 #include "btstack_uart_block.h"
 
+#include <inttypes.h>
+
 #define ENABLE_LOG_EHCILL
 
 #ifdef ENABLE_EHCILL
@@ -111,6 +113,7 @@ typedef enum {
     H4_W4_EVENT_HEADER,
     H4_W4_ACL_HEADER,
     H4_W4_SCO_HEADER,
+    H4_W4_ISO_HEADER,
     H4_W4_PAYLOAD,
 } H4_STATE;
 
@@ -126,8 +129,8 @@ typedef enum {
 } TX_STATE;
 
 // UART Driver + Config
-static const btstack_uart_block_t * btstack_uart;
-static btstack_uart_config_t uart_config;
+static const btstack_uart_t * btstack_uart;
+static btstack_uart_config_t hci_transport_h4_uart_config;
 
 // write state
 static TX_STATE tx_state;         
@@ -136,7 +139,7 @@ static uint8_t * ehcill_tx_data;
 static uint16_t  ehcill_tx_len;   // 0 == no outgoing packet
 #endif
 
-static  void (*packet_handler)(uint8_t packet_type, uint8_t *packet, uint16_t size) = dummy_handler;
+static void (*hci_transport_h4_packet_handler)(uint8_t packet_type, uint8_t *packet, uint16_t size) = dummy_handler;
 
 // packet reader state machine
 static  H4_STATE h4_state;
@@ -215,11 +218,11 @@ static void hci_transport_h4_packet_complete(void){
 #endif
             }
 #endif
-    uint16_t packet_len = read_pos-1;
+    uint16_t packet_len = read_pos-1u;
 
     // reset state machine before delivering packet to stack as it might close the transport
     hci_transport_h4_reset_statemachine();
-    packet_handler(hci_packet[0], &hci_packet[1], packet_len);
+    hci_transport_h4_packet_handler(hci_packet[0], &hci_packet[1], packet_len);
 }
 
 static void hci_transport_h4_block_read(void){
@@ -241,6 +244,12 @@ static void hci_transport_h4_block_read(void){
                     bytes_to_read = HCI_SCO_HEADER_SIZE;
                     h4_state = H4_W4_SCO_HEADER;
                     break;
+#ifdef ENABLE_LE_ISOCHRONOUS_STREAMS
+                case HCI_ISO_DATA_PACKET:
+                    bytes_to_read = HCI_ISO_HEADER_SIZE;
+                    h4_state = H4_W4_SCO_HEADER;
+                    break;
+#endif
 #ifdef ENABLE_EHCILL
                 case EHCILL_GO_TO_SLEEP_IND:
                 case EHCILL_GO_TO_SLEEP_ACK:
@@ -290,6 +299,17 @@ static void hci_transport_h4_block_read(void){
             h4_state = H4_W4_PAYLOAD;
             break;
 
+        case H4_W4_ISO_HEADER:
+            bytes_to_read = little_endian_read_16( hci_packet, 3) & 0x3fff;
+            // check ISO length
+            if (bytes_to_read > (HCI_INCOMING_PACKET_BUFFER_SIZE - HCI_ACL_HEADER_SIZE)){
+                log_error("hci_transport_h4: invalid ISO payload len %d - only space for %u", bytes_to_read, HCI_INCOMING_PACKET_BUFFER_SIZE - HCI_ISO_HEADER_SIZE);
+                hci_transport_h4_reset_statemachine();
+                break;
+            }
+            h4_state = H4_W4_PAYLOAD;
+            break;
+
         case H4_W4_PAYLOAD:
             hci_transport_h4_packet_complete();
             break;
@@ -297,6 +317,9 @@ static void hci_transport_h4_block_read(void){
         case H4_OFF:
             bytes_to_read = 0;
             break;
+        default:
+            btstack_assert(false);
+            break; 
     }
 
 #ifdef ENABLE_BAUDRATE_CHANGE_FLOWCONTROL_BUG_WORKAROUND
@@ -309,7 +332,7 @@ static void hci_transport_h4_block_read(void){
 #endif
 
     // forward packet if payload size == 0
-    if (h4_state == H4_W4_PAYLOAD && bytes_to_read == 0) {
+    if (h4_state == H4_W4_PAYLOAD && bytes_to_read == 0u) {
         hci_transport_h4_packet_complete();
     }
 
@@ -335,7 +358,7 @@ static void hci_transport_h4_block_sent(void){
             hci_transport_h4_ehcill_handle_packet_sent();
 #endif
             // notify upper stack that it can send again
-            packet_handler(HCI_EVENT_PACKET, (uint8_t *) &packet_sent_event[0], sizeof(packet_sent_event));
+            hci_transport_h4_packet_handler(HCI_EVENT_PACKET, (uint8_t *) &packet_sent_event[0], sizeof(packet_sent_event));
             break;
 
 #ifdef ENABLE_EHCILL        
@@ -358,13 +381,13 @@ static int hci_transport_h4_can_send_now(uint8_t packet_type){
 static int hci_transport_h4_send_packet(uint8_t packet_type, uint8_t * packet, int size){
 
     // store packet type before actual data and increase size
-    size++;
-    packet--;
-    *packet = packet_type;
+    uint8_t * buffer = &packet[-1];
+    uint32_t  buffer_size = size + 1;
+    buffer[0] = packet_type;
 
 #ifdef ENABLE_BAUDRATE_CHANGE_FLOWCONTROL_BUG_WORKAROUND
     if ((baudrate_change_workaround_state == BAUDRATE_CHANGE_WORKAROUND_CHIPSET_DETECTED)
-    && (memcmp(packet, baud_rate_command_prefix, sizeof(baud_rate_command_prefix)) == 0)) {
+    && (memcmp(buffer, baud_rate_command_prefix, sizeof(baud_rate_command_prefix)) == 0)) {
         log_info("Baud rate command detected, expect command complete event next");
         baudrate_change_workaround_state = BAUDRATE_CHANGE_WORKAROUND_BAUDRATE_COMMAND_SENT;
     }
@@ -372,8 +395,8 @@ static int hci_transport_h4_send_packet(uint8_t packet_type, uint8_t * packet, i
 
 #ifdef ENABLE_EHCILL
     // store request for later
-    ehcill_tx_len   = size;
-    ehcill_tx_data  = packet;
+    ehcill_tx_len   = buffer_size;
+    ehcill_tx_data  = buffer;
     switch (ehcill_state){
         case EHCILL_STATE_SLEEP:
             hci_transport_h4_ehcill_trigger_wakeup();
@@ -388,7 +411,7 @@ static int hci_transport_h4_send_packet(uint8_t packet_type, uint8_t * packet, i
 
     // start sending
     tx_state = TX_W4_PACKET_SENT;
-    btstack_uart->send_block(packet, size);
+    btstack_uart->send_block(buffer, buffer_size);
     return 0;
 }
 
@@ -405,16 +428,17 @@ static void hci_transport_h4_init(const void * transport_config){
 
     // extract UART config from transport config
     hci_transport_config_uart_t * hci_transport_config_uart = (hci_transport_config_uart_t*) transport_config;
-    uart_config.baudrate    = hci_transport_config_uart->baudrate_init;
-    uart_config.flowcontrol = hci_transport_config_uart->flowcontrol;
-    uart_config.device_name = hci_transport_config_uart->device_name;
+    hci_transport_h4_uart_config.baudrate    = hci_transport_config_uart->baudrate_init;
+    hci_transport_h4_uart_config.flowcontrol = hci_transport_config_uart->flowcontrol;
+    hci_transport_h4_uart_config.parity      = hci_transport_config_uart->parity;
+    hci_transport_h4_uart_config.device_name = hci_transport_config_uart->device_name;
 
     // set state to off
     tx_state = TX_OFF;
     h4_state = H4_OFF;
 
     // setup UART driver
-    btstack_uart->init(&uart_config);
+    btstack_uart->init(&hci_transport_h4_uart_config);
     btstack_uart->set_block_received(&hci_transport_h4_block_read);
     btstack_uart->set_block_sent(&hci_transport_h4_block_sent);
 }
@@ -422,7 +446,7 @@ static void hci_transport_h4_init(const void * transport_config){
 static int hci_transport_h4_open(void){
     // open uart driver
     int res = btstack_uart->open();
-    if (res){
+    if (res != 0){
         return res;
     }
 
@@ -447,7 +471,7 @@ static int hci_transport_h4_close(void){
 }
 
 static void hci_transport_h4_register_packet_handler(void (*handler)(uint8_t packet_type, uint8_t *packet, uint16_t size)){
-    packet_handler = handler;
+    hci_transport_h4_packet_handler = handler;
 }
 
 static void dummy_handler(uint8_t packet_type, uint8_t *packet, uint16_t size){
@@ -472,7 +496,7 @@ static void hci_transport_h4_ehcill_emit_sleep_state(int sleep_active){
     event[0] = HCI_EVENT_TRANSPORT_SLEEP_MODE;
     event[1] = sizeof(event) - 2;
     event[2] = sleep_active;
-    packet_handler(HCI_EVENT_PACKET, &event[0], sizeof(event));        
+    hci_transport_h4_packet_handler(HCI_EVENT_PACKET, &event[0], sizeof(event));        
 }
 
 static void hci_transport_h4_ehcill_wakeup_handler(void){
@@ -725,21 +749,26 @@ static void hci_transport_h4_ehcill_handle_ehcill_command_sent(void){
 
 
 // configure and return h4 singleton
-const hci_transport_t * hci_transport_h4_instance(const btstack_uart_block_t * uart_driver) {
-
-    static const hci_transport_t hci_transport_h4 = {
-            /* const char * name; */                                        "H4",
-            /* void   (*init) (const void *transport_config); */            &hci_transport_h4_init,
-            /* int    (*open)(void); */                                     &hci_transport_h4_open,
-            /* int    (*close)(void); */                                    &hci_transport_h4_close,
-            /* void   (*register_packet_handler)(void (*handler)(...); */   &hci_transport_h4_register_packet_handler,
-            /* int    (*can_send_packet_now)(uint8_t packet_type); */       &hci_transport_h4_can_send_now,
-            /* int    (*send_packet)(...); */                               &hci_transport_h4_send_packet,
-            /* int    (*set_baudrate)(uint32_t baudrate); */                &hci_transport_h4_set_baudrate,
-            /* void   (*reset_link)(void); */                               NULL,
-            /* void   (*set_sco_config)(uint16_t voice_setting, int num_connections); */ NULL,
-    };
+static const hci_transport_t hci_transport_h4 = {
+        /* const char * name; */                                        "H4",
+        /* void   (*init) (const void *transport_config); */            &hci_transport_h4_init,
+        /* int    (*open)(void); */                                     &hci_transport_h4_open,
+        /* int    (*close)(void); */                                    &hci_transport_h4_close,
+        /* void   (*register_packet_handler)(void (*handler)(...); */   &hci_transport_h4_register_packet_handler,
+        /* int    (*can_send_packet_now)(uint8_t packet_type); */       &hci_transport_h4_can_send_now,
+        /* int    (*send_packet)(...); */                               &hci_transport_h4_send_packet,
+        /* int    (*set_baudrate)(uint32_t baudrate); */                &hci_transport_h4_set_baudrate,
+        /* void   (*reset_link)(void); */                               NULL,
+        /* void   (*set_sco_config)(uint16_t voice_setting, int num_connections); */ NULL,
+};
 
+const hci_transport_t * hci_transport_h4_instance_for_uart(const btstack_uart_t * uart_driver){
     btstack_uart = uart_driver;
     return &hci_transport_h4;
 }
+
+// @deprecated
+const hci_transport_t * hci_transport_h4_instance(const btstack_uart_block_t * uart_driver) {
+    btstack_uart = (const btstack_uart_t *) uart_driver;
+    return &hci_transport_h4;
+}
