@@ -20,8 +20,8 @@
  * THIS SOFTWARE IS PROVIDED BY BLUEKITCHEN GMBH AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATTHIAS
- * RINGWALD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL BLUEKITCHEN
+ * GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
@@ -68,6 +68,7 @@
 #include "mesh/pb_gatt.h"
 #include "mesh/provisioning.h"
 #include "mesh/provisioning_device.h"
+#include "mesh/provisioning_provisioner.h"
 
 static void mesh_node_store_provisioning_data(mesh_provisioning_data_t * provisioning_data);
 static int mesh_node_startup_from_tlv(void);
@@ -299,39 +300,37 @@ static void mesh_provisioning_message_handler (uint8_t packet_type, uint16_t cha
 static void hci_packet_handler (uint8_t packet_type, uint16_t channel, uint8_t *packet, uint16_t size){
     UNUSED(channel);
     UNUSED(size);
+    
+    if (packet_type != HCI_EVENT_PACKET) return;
 
-    switch (packet_type) {
-        case HCI_EVENT_PACKET:
-            switch (hci_event_packet_get_type(packet)) {
-                case BTSTACK_EVENT_STATE:
-                    if (btstack_event_state_get_state(packet) != HCI_STATE_WORKING) break;
-                    // get TLV instance
-                    btstack_tlv_get_instance(&btstack_tlv_singleton_impl, &btstack_tlv_singleton_context);
+    switch (hci_event_packet_get_type(packet)) {
+        case BTSTACK_EVENT_STATE:
+            if (btstack_event_state_get_state(packet) != HCI_STATE_WORKING) break;
+            // get TLV instance
+            btstack_tlv_get_instance(&btstack_tlv_singleton_impl, &btstack_tlv_singleton_context);
 
-                    // startup from static provisioning data stored in TLV
-                    provisioned = mesh_node_startup_from_tlv();
-                    break;
-                
+            // startup from static provisioning data stored in TLV
+            provisioned = mesh_node_startup_from_tlv();
+            break;
+        
 #ifdef ENABLE_MESH_PROXY_SERVER
-                case HCI_EVENT_DISCONNECTION_COMPLETE:
-                    // enable PB_GATT
-                    if (provisioned == 0){
-                        mesh_proxy_start_advertising_unprovisioned_device();
-                    } else {
-                        mesh_proxy_start_advertising_with_network_id();
-                    }
-                    break;
-                    
-                case HCI_EVENT_LE_META:
-                    if (hci_event_le_meta_get_subevent_code(packet) !=  HCI_SUBEVENT_LE_CONNECTION_COMPLETE) break;
-                    // disable PB_GATT
-                    mesh_proxy_stop_advertising_unprovisioned_device();
-                    break;
-#endif
-                default:
-                    break;
+        case HCI_EVENT_DISCONNECTION_COMPLETE:
+            // enable PB_GATT
+            if (provisioned == 0){
+                mesh_proxy_start_advertising_unprovisioned_device();
+            } else {
+                mesh_proxy_start_advertising_with_network_id();
             }
             break;
+            
+        case HCI_EVENT_LE_META:
+            if (hci_event_le_meta_get_subevent_code(packet) !=  HCI_SUBEVENT_LE_CONNECTION_COMPLETE) break;
+            // disable PB_GATT
+            mesh_proxy_stop_advertising_unprovisioned_device();
+            break;
+#endif
+        default:
+            break;
     }
 }
 
@@ -1009,18 +1008,6 @@ static void mesh_access_secure_network_beacon_handler(uint8_t packet_type, uint1
 
 static const uint32_t mesh_tag_for_prov_data = ((uint32_t) 'P' << 24) | ((uint32_t) 'R' << 16) | ((uint32_t) 'O' <<  8) | (uint32_t)'V';
 
-void mesh_node_reset(void){
-    // PROV
-    btstack_tlv_singleton_impl->delete_tag(btstack_tlv_singleton_context, mesh_tag_for_prov_data);
-    // everything else
-    mesh_delete_network_keys();
-    mesh_delete_app_keys();
-    mesh_delete_appkey_lists();
-    mesh_delete_virtual_addresses();
-    mesh_delete_subscriptions();
-    mesh_delete_publications();
-}
-
 typedef struct {
     uint16_t unicast_address;
     uint8_t  flags;
@@ -1069,6 +1056,23 @@ static void mesh_access_setup_without_provisiong_data_random(void * arg){
     mesh_access_setup_unprovisioned_device(random_device_uuid);
 }
 
+void mesh_node_reset(void){
+    // PROV
+    btstack_tlv_singleton_impl->delete_tag(btstack_tlv_singleton_context, mesh_tag_for_prov_data);
+    // everything else
+    mesh_delete_network_keys();
+    mesh_delete_app_keys();
+    mesh_delete_appkey_lists();
+    mesh_delete_virtual_addresses();
+    mesh_delete_subscriptions();
+    mesh_delete_publications();
+    // also reset iv index + sequence number
+    mesh_set_iv_index(0);
+    mesh_sequence_number_set(0);
+    // start advertising as unprovisioned device
+    mesh_access_setup_unprovisioned_device(mesh_node_get_device_uuid());
+}
+
 static void mesh_access_setup_with_provisiong_data_random(void * arg){
     UNUSED(arg);
     // set random value
@@ -1163,10 +1167,14 @@ static int mesh_node_startup_from_tlv(void){
     return prov_data_valid;
 }
 
-static void mesh_control_message_handler(mesh_pdu_t * pdu){
+static void mesh_control_message_handler(mesh_transport_callback_type_t callback_type, mesh_transport_status_t status, mesh_pdu_t * pdu){
+    UNUSED(callback_type);
+    UNUSED(status);
+
     // get opcode 
     uint8_t opcode = mesh_pdu_control_opcode(pdu);
-    printf("Opcode: 0x%02x\n", opcode);
+    printf("MESH Control Message, Opcode: 0x%02x + ", opcode);
+    printf_hexdump(mesh_pdu_data(pdu), mesh_pdu_len(pdu));
 
     uint8_t init_ttl;
     uint8_t hops = 0;
@@ -1225,6 +1233,11 @@ void mesh_init(void){
     provisioning_device_init();
     provisioning_device_register_packet_handler(&mesh_provisioning_message_handler);
 
+#ifdef ENABLE_MESH_PROVISIONER
+    provisioning_provisioner_init();
+    provisioning_provisioner_register_packet_handler(&mesh_provisioning_message_handler);
+#endif
+
     // Node Configuration
     mesh_node_init();
 
