@@ -35,19 +35,11 @@
  *
  */
 
-#define __BTSTACK_FILE__ "hid_keyboard_demo.c"
- 
-// *****************************************************************************
-/* EXAMPLE_START(hid_keyboard_demo): HID Keyboard (Server) Demo
- *
- * @text This HID Device example demonstrates how to implement
- * an HID keyboard. Without a HAVE_BTSTACK_STDIN, a fixed demo text is sent
- * If HAVE_BTSTACK_STDIN is defined, you can type from the terminal
- *
- * @text Status: Basic implementation. HID Request from Host are not answered yet. Works with iOS.
- */
-// *****************************************************************************
+#define BTSTACK_FILE__ "hid_device_test.c"
 
+/*
+ * hid_device_test.c : Tool for testig HID device with PTS, see hid_device_test.md for PTS tests command sequences
+ */ 
 
 #include <stdint.h>
 #include <stdio.h>
@@ -61,7 +53,7 @@
 #include "btstack_stdin.h"
 #endif
 
-// #define REPORT_ID_DECLARED 
+#define REPORT_ID_DECLARED 
 // to enable demo text on POSIX systems
 // #undef HAVE_BTSTACK_STDIN
 
@@ -127,65 +119,26 @@ const uint8_t hid_descriptor_keyboard_boot_mode[] = {
 #define CHAR_TAB         '\t'
 #define CHAR_BACKSPACE   0x7f
 
-// Simplified US Keyboard with Shift modifier
-
-/**
- * English (US)
- */
-static const uint8_t keytable_us_none [] = {
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /*   0-3 */
-    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',                   /*  4-13 */
-    'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',                   /* 14-23 */
-    'u', 'v', 'w', 'x', 'y', 'z',                                       /* 24-29 */
-    '1', '2', '3', '4', '5', '6', '7', '8', '9', '0',                   /* 30-39 */
-    CHAR_RETURN, CHAR_ESCAPE, CHAR_BACKSPACE, CHAR_TAB, ' ',            /* 40-44 */
-    '-', '=', '[', ']', '\\', CHAR_ILLEGAL, ';', '\'', 0x60, ',',       /* 45-54 */
-    '.', '/', CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,   /* 55-60 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 61-64 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 65-68 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 69-72 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 73-76 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 77-80 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 81-84 */
-    '*', '-', '+', '\n', '1', '2', '3', '4', '5',                       /* 85-97 */
-    '6', '7', '8', '9', '0', '.', 0xa7,                                 /* 97-100 */
-}; 
-
-static const uint8_t keytable_us_shift[] = {
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /*  0-3  */
-    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',                   /*  4-13 */
-    'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',                   /* 14-23 */
-    'U', 'V', 'W', 'X', 'Y', 'Z',                                       /* 24-29 */
-    '!', '@', '#', '$', '%', '^', '&', '*', '(', ')',                   /* 30-39 */
-    CHAR_RETURN, CHAR_ESCAPE, CHAR_BACKSPACE, CHAR_TAB, ' ',            /* 40-44 */
-    '_', '+', '{', '}', '|', CHAR_ILLEGAL, ':', '"', 0x7E, '<',         /* 45-54 */
-    '>', '?', CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,   /* 55-60 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 61-64 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 65-68 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 69-72 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 73-76 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 77-80 */
-    CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL, CHAR_ILLEGAL,             /* 81-84 */
-    '*', '-', '+', '\n', '1', '2', '3', '4', '5',                       /* 85-97 */
-    '6', '7', '8', '9', '0', '.', 0xb1,                                 /* 97-100 */
-}; 
-
 // STATE
 
-static uint8_t hid_service_buffer[250];
+static uint8_t hid_service_buffer[300];
 static uint8_t device_id_sdp_service_buffer[100];
 static const char hid_device_name[] = "BTstack HID Keyboard";
 static btstack_packet_callback_registration_t hci_event_callback_registration;
 static uint16_t hid_cid;
+static hci_con_handle_t hid_con_handle = HCI_CON_HANDLE_INVALID;
 static bd_addr_t device_addr;
 static int     report_data_ready = 1;
 static uint8_t report_data[20];
-static uint8_t hid_boot_device = 1;
-static int send_mouse_on_interrupt_channel = 0;
-static int send_keyboard_on_interrupt_channel = 0;
+static bool    hid_boot_device = true;
+static bool    send_mouse_on_interrupt_channel = false;
+static bool    send_keyboard_on_interrupt_channel = false;
+
+static uint16_t host_max_latency = 1600;
+static uint16_t host_min_timeout = 3200;
 
 #ifdef HAVE_BTSTACK_STDIN
-static const char * device_addr_string = "BC:EC:5D:E6:15:03";
+static const char * device_addr_string = "00:1B:DC:08:E2:5C";
 #endif
 
 static enum {
@@ -195,60 +148,43 @@ static enum {
     APP_CONNECTED
 } app_state = APP_BOOTING;
 
-// static uint8_t modifier = 0;
-// static uint8_t keycode = 0;
-// static uint8_t report[] = { /* 0xa1, */ modifier, 0, 0, keycode, 0, 0, 0, 0, 0};
-
-// HID Keyboard lookup
-static int lookup_keycode(uint8_t character, const uint8_t * table, int size, uint8_t * keycode){
-    int i;
-    for (i=0;i<size;i++){
-        if (table[i] != character) continue;
-        *keycode = i;
-        return 1;
-    }
-    return 0;
-}
-
-static int keycode_and_modifer_us_for_character(uint8_t character, uint8_t * keycode, uint8_t * modifier){
-    int found;
-    found = lookup_keycode(character, keytable_us_none, sizeof(keytable_us_none), keycode);
-    if (found) {
-        *modifier = 0;  // none
-        return 1;
-    }
-    found = lookup_keycode(character, keytable_us_shift, sizeof(keytable_us_shift), keycode);
-    if (found) {
-        *modifier = 2;  // shift
-        return 1;
-    }
-    return 0;
-}
+static bool virtual_cable_enabled  = false;
+static bool virtual_cable_unplugged = false;
 
 // HID Report sending
 static int send_keycode;
 static int send_modifier;
 
-static void send_key(int modifier, int keycode){
-    send_keycode = keycode;
-    send_modifier = modifier;
-    hid_device_request_can_send_now_event(hid_cid);
-}
-
 static void send_keyboard_report_on_interrupt_channel(int modifier, int keycode){
     uint8_t report[] = {0xa1, 
+#ifdef REPORT_ID_DECLARED
         0x01, 
+#endif
         modifier, 0, 0, keycode, 0, 0, 0, 0};
     hid_device_send_interrupt_message(hid_cid, &report[0], sizeof(report));
 }
 
 static void send_mouse_report_on_interrupt_channel(uint8_t buttons, int8_t dx, int8_t dy){
     uint8_t report[] = {0xa1, 
+#ifdef REPORT_ID_DECLARED
         0x02, 
+#endif
         buttons, dx, dy, 0};
     hid_device_send_interrupt_message(hid_cid, &report[0], sizeof(report));
 }
 
+/**
+ * @brief Initiate a Virtual Cable unplug from the already pluged remote device. Reconnection requests after unplug will be rejected.
+ * @param  hid_cid
+ * @return status 
+ */
+static uint8_t hid_device_send_virtual_cable_unplug(uint16_t hcid){
+    if (!virtual_cable_enabled) return ERROR_CODE_COMMAND_DISALLOWED;
+    uint8_t report[] = { (HID_MESSAGE_TYPE_HID_CONTROL << 4) | HID_CONTROL_PARAM_VIRTUAL_CABLE_UNPLUG };
+    hid_device_send_control_message(hcid, &report[0], sizeof(report));
+    return ERROR_CODE_SUCCESS;
+}
+
 // HID Report sending
 static int prepare_mouse_report(hid_report_type_t report_type, uint8_t buttons, int8_t dx, int8_t dy, uint8_t * buffer, int buffer_size){
     UNUSED(report_type);
@@ -329,34 +265,43 @@ static void hid_report_data_callback(uint16_t cid, hid_report_type_t report_type
     printf("do smth with report\n");
 }
 
-#ifdef HAVE_BTSTACK_STDIN
 
-// On systems with STDIN, we can directly type on the console
-#if 0
 static void show_usage(void){
     bd_addr_t      iut_address;
     gap_local_bd_addr(iut_address);
     printf("\n--- Bluetooth HID Host Test Console %s ---\n", bd_addr_to_str(iut_address));
-    printf("c      - connect \n");
-    printf("l      - set limited discoverable mode\n");
-    printf("L      - restset limited discoverable mode\n");
+    printf("c      - Connect to %s...\n", bd_addr_to_str(device_addr));
+    printf("D      - Disconnect\n");
+    printf("I      - Disconnect from intrrupt channel %s...\n", bd_addr_to_str(device_addr));
+    printf("C      - Disconnect from control channel %s...\n", bd_addr_to_str(device_addr));
+    printf("\n");
+    
+    printf("l      - Set limited discoverable mode\n");
+    printf("m      - Request can send now (mouse)\n");
+    printf("M      - Request can send now (keyboard)\n");
+    printf("L      - Reset limited discoverable mode\n");
+    printf("u      - Unplug\n");
+    printf("w      - Set sniff subrating\n");
+    printf("z      - Enter sniff mode\n");
+    printf("q      - Set QoS 'Guaranteed'\n");
+
     printf("Ctrl-c - exit\n");
     printf("---\n");
 }
-#endif
 
 static void stdin_process(char character){
-    uint8_t modifier;
-    uint8_t keycode;
-    int found;
     switch (character){
         case 'D':
             printf("Disconnect from %s...\n", bd_addr_to_str(device_addr));
             hid_device_disconnect(hid_cid);
             break;
         case 'c':
-            printf("Connecting %s...\n", bd_addr_to_str(device_addr));
-            hid_device_connect(device_addr, &hid_cid);
+            if (virtual_cable_unplugged){
+                printf("Cannot connect to %s, device has been unplugged...\n", bd_addr_to_str(device_addr));
+            } else {
+                printf("Connecting %s...\n", bd_addr_to_str(device_addr));
+                hid_device_connect(device_addr, &hid_cid);
+            }
             return;
         case 'I':
             printf("Disconnect from intrrupt channel %s...\n", bd_addr_to_str(device_addr));
@@ -373,87 +318,65 @@ static void stdin_process(char character){
             hci_send_cmd(&hci_write_current_iac_lap_two_iacs, 2, GAP_IAC_GENERAL_INQUIRY, GAP_IAC_LIMITED_INQUIRY);
             return;
         case 'm':
-            send_mouse_on_interrupt_channel = 1;
+            send_mouse_on_interrupt_channel = true;
             hid_device_request_can_send_now_event(hid_cid);
             break;
         case 'M':
-            send_keyboard_on_interrupt_channel = 1;
+            send_keyboard_on_interrupt_channel = true;
             hid_device_request_can_send_now_event(hid_cid);
             break;
         case 'L':
-            gap_discoverable_control(0);
             printf("reset limited discoverable mode\n");
+            gap_discoverable_control(0);
             return;
+        case 'u':
+            printf("Send unplug request to %s...\n", bd_addr_to_str(device_addr));
+            hid_device_send_virtual_cable_unplug(hid_cid);
+            break;
+
+        case 'w':
+            printf("Set sniff subrating\n");
+            gap_sniff_subrating_configure(hid_con_handle, 0, 0, 0);
+            break;
+
+        case 'z':{
+            printf("Enter sniff mode \n");
+            uint16_t sniff_min_interval = 18;
+            uint16_t sniff_max_interval = 18;
+            uint16_t sniff_attempt = 4;
+            uint16_t sniff_timeout = 2;
+            gap_sniff_mode_enter(hid_con_handle, sniff_min_interval, sniff_max_interval, sniff_attempt, sniff_timeout);
+            break;
+        }
+        case 'q':
+            printf("Set QoS 'Guaranteed'\n");
+            gap_qos_set(hid_con_handle, 1, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff);
+            break;
+
         case '\n':
         case '\r':
             break;
+
         default:
-            // show_usage();
+            show_usage();
             break;
     }
 
     switch (app_state){
         case APP_BOOTING:
         case APP_CONNECTING:
+        case APP_CONNECTED:
             // ignore
             break;
 
-        case APP_CONNECTED:
-            found = keycode_and_modifer_us_for_character(character, &keycode, &modifier);
-            if (found){
-                send_key(modifier, keycode);
-                return;
-            }
-            break;
         case APP_NOT_CONNECTED:
             printf("Connecting to %s...\n", bd_addr_to_str(device_addr));
             hid_device_connect(device_addr, &hid_cid);
             break;
     }
 }
-#else
 
-// On embedded systems, send constant demo text with fixed period
 
-#define TYPING_PERIOD_MS 100
-static const char * demo_text = "\n\nHello World!\n\nThis is the BTstack HID Keyboard Demo running on an Embedded Device.\n\n";
-
-static int demo_pos;
-static btstack_timer_source_t typing_timer;
-
-static void typing_timer_handler(btstack_timer_source_t * ts){
-
-    // abort if not connected
-    if (!hid_cid) return;
-
-    // get next character
-    uint8_t character = demo_text[demo_pos++];
-    if (demo_text[demo_pos] == 0){
-        demo_pos = 0;
-    }
-
-    // get keycodeand send
-    uint8_t modifier;
-    uint8_t keycode;
-    int found = keycode_and_modifer_us_for_character(character, &keycode, &modifier);
-    if (found){
-        send_key(modifier, keycode);
-    }
-
-    // set next timer
-    btstack_run_loop_set_timer(ts, TYPING_PERIOD_MS);
-    btstack_run_loop_add_timer(ts);
-}
-
-static void hid_embedded_start_typing(void){
-    demo_pos = 0;
-    // set one-shot timer
-    typing_timer.process = &typing_timer_handler;
-    btstack_run_loop_set_timer(&typing_timer, TYPING_PERIOD_MS);
-    btstack_run_loop_add_timer(&typing_timer);
-}
-
-#endif
 
 static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * packet, uint16_t packet_size){
     UNUSED(channel);
@@ -461,7 +384,7 @@ static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * pack
     uint8_t status;
     switch (packet_type){
         case HCI_EVENT_PACKET:
-            switch (packet[0]){
+            switch (hci_event_packet_get_type(packet)){
                 case BTSTACK_EVENT_STATE:
                     if (btstack_event_state_get_state(packet) != HCI_STATE_WORKING) return;
                     app_state = APP_NOT_CONNECTED;
@@ -485,17 +408,21 @@ static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * pack
                                 return;
                             }
                             app_state = APP_CONNECTED;
+                            virtual_cable_unplugged = false;
+
                             hid_subevent_connection_opened_get_bd_addr(packet, device_addr);
                             hid_cid = hid_subevent_connection_opened_get_hid_cid(packet);
-#ifdef HAVE_BTSTACK_STDIN                        
+                            hid_con_handle = hid_subevent_connection_opened_get_con_handle(packet);
                             printf("HID Connected, please start typing... %s\n", bd_addr_to_str(device_addr));
-#else                        
-                            printf("HID Connected, sending demo text...\n");
-                            hid_embedded_start_typing();
-#endif
                             break;
                         case HID_SUBEVENT_CONNECTION_CLOSED:
                             printf("HID Disconnected\n");
+                            if (virtual_cable_enabled){
+                                virtual_cable_unplugged = true;
+                                gap_drop_link_key_for_bd_addr(device_addr);
+                                gap_disconnect(hid_con_handle);
+                            }
+                            hid_con_handle = HCI_CON_HANDLE_INVALID;
                             app_state = APP_NOT_CONNECTED;
                             hid_cid = 0;
                             break;
@@ -506,19 +433,26 @@ static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * pack
                         case HID_SUBEVENT_EXIT_SUSPEND:
                             printf("HID Exit Suspend\n");
                             break;
-                            
+                        case HID_SUBEVENT_VIRTUAL_CABLE_UNPLUG:
+                            if (virtual_cable_enabled){
+                                printf("HID Device is unplugged.\n");
+                                virtual_cable_unplugged = true;
+                                gap_drop_link_key_for_bd_addr(device_addr);
+                            }
+                            break;
+
                         case HID_SUBEVENT_CAN_SEND_NOW:
                             if (send_mouse_on_interrupt_channel){
-                                send_mouse_on_interrupt_channel = 0;
-                                printf("send mouse on interrupt channel\n");
+                                send_mouse_on_interrupt_channel = false;
                                 send_mouse_report_on_interrupt_channel(0,0,0);
                                 break;     
                             }
                             if (send_keyboard_on_interrupt_channel){
-                                send_keyboard_on_interrupt_channel = 0;
+                                send_keyboard_on_interrupt_channel = false;
                                 send_keyboard_report_on_interrupt_channel(send_modifier, send_keycode);
                                 send_keycode = 0;
                                 send_modifier = 0;
+                                break;
                             }
                         
                             if (send_keycode){
@@ -543,17 +477,16 @@ static void packet_handler(uint8_t packet_type, uint16_t channel, uint8_t * pack
     }
 }
 
+static int hid_device_connection_filter(bd_addr_t addr, hci_link_type_t link_type){
+    UNUSED(link_type);
+    if (virtual_cable_enabled && !virtual_cable_unplugged && (memcmp(addr, device_addr, 6) != 0)){
+        printf("Virtual Cable: reject incoming connection from %s\n", bd_addr_to_str(addr));
+        return 0;
+    }
+    return 1;
+}
 
 
-/* @section Main Application Setup
- *
- * @text Listing MainConfiguration shows main application code. 
- * To run a HID Device service you need to initialize the SDP, and to create and register HID Device record with it. 
- * At the end the Bluetooth stack is started.
- */
-
-/* LISTING_START(MainConfiguration): Setup HID Device */
-
 int btstack_main(int argc, const char * argv[]);
 int btstack_main(int argc, const char * argv[]){
     (void)argc;
@@ -561,8 +494,11 @@ int btstack_main(int argc, const char * argv[]){
 
     gap_discoverable_control(1);
     gap_set_class_of_device(0x2540);
-    gap_set_local_name("HID Keyboard Demo 00:00:00:00:00:00");
+    gap_set_local_name("HID Device 00:00:00:00:00:00");
     
+    // Allow sniff mode requests by HID device and support role switch
+    gap_set_default_link_policy_settings(LM_LINK_POLICY_ENABLE_SNIFF_MODE | LM_LINK_POLICY_ENABLE_ROLE_SWITCH);
+
     // L2CAP
     l2cap_init();
 
@@ -570,12 +506,31 @@ int btstack_main(int argc, const char * argv[]){
     sdp_init();
     memset(hid_service_buffer, 0, sizeof(hid_service_buffer));
 
-    uint8_t hid_reconnect_initiate = 1;
     uint8_t hid_virtual_cable = 1;
-    // hid sevice subclass 2540 Keyboard, hid counntry code 33 US, hid virtual cable on, hid reconnect initiate on, hid boot device off 
-    hid_create_sdp_record(hid_service_buffer, 0x10001, 0x2540, 33, 
-        hid_virtual_cable, hid_reconnect_initiate, hid_boot_device, 
-        hid_descriptor_keyboard_boot_mode, sizeof(hid_descriptor_keyboard_boot_mode), hid_device_name);
+    uint8_t hid_remote_wake = 1;
+    uint8_t hid_reconnect_initiate = 1;
+    uint8_t hid_normally_connectable = 1;
+
+    if (hid_virtual_cable == 1){
+        printf("Virtual Cable enabled, accept only connections from %s\n", device_addr_string);
+        virtual_cable_enabled = true;
+        virtual_cable_unplugged = false;
+    }
+
+    hid_sdp_record_t hid_params = {
+        // hid sevice subclass 2540 Keyboard, hid counntry code 33 US
+        0x2540, 33, 
+        hid_virtual_cable, hid_remote_wake, 
+        hid_reconnect_initiate, hid_normally_connectable,
+        hid_boot_device, 
+        host_max_latency, host_min_timeout, 
+        3200,
+        hid_descriptor_keyboard_boot_mode,
+        sizeof(hid_descriptor_keyboard_boot_mode), 
+        hid_device_name
+    };
+
+    hid_create_sdp_record(hid_service_buffer, 0x10001, &hid_params);
 
     printf("HID service record size: %u\n", de_get_len( hid_service_buffer));
     sdp_register_service(hid_service_buffer);
@@ -596,6 +551,8 @@ int btstack_main(int argc, const char * argv[]){
     hid_device_register_report_request_callback(&hid_get_report_callback);
     hid_device_register_set_report_callback(&hid_set_report_callback);
     hid_device_register_report_data_callback(&hid_report_data_callback);
+    
+    gap_register_classic_connection_filter(&hid_device_connection_filter);
 
     sscanf_bd_addr(device_addr_string, device_addr);
     btstack_stdin_setup(stdin_process);
@@ -603,5 +560,4 @@ int btstack_main(int argc, const char * argv[]){
     hci_power_control(HCI_POWER_ON);
     return 0;
 }
-/* LISTING_END */
-/* EXAMPLE_END */
+
