@@ -35,7 +35,7 @@
  *
  */
 
-#define __BTSTACK_FILE__ "gatt_heart_rate_client.c"
+#define BTSTACK_FILE__ "gatt_heart_rate_client.c"
 
 // *****************************************************************************
 /* EXAMPLE_START(gatt_heart_rate_client): Connects to Heart Rate Sensor and reports measurements */
