@@ -76,7 +76,7 @@ static inline uint32_t us_to_ms(uint64_t us) {
  * \ingroup timestamp
  * \brief Convert a timestamp into a number of milliseconds since boot.
  * \param t an absolute_time_t value to convert
- * \return the number of microseconds since boot represented by t
+ * \return the number of milliseconds since boot represented by t
  * \sa to_us_since_boot()
  */
 static inline uint32_t to_ms_since_boot(absolute_time_t t) {
@@ -95,8 +95,9 @@ static inline absolute_time_t delayed_by_us(const absolute_time_t t, uint64_t us
     absolute_time_t t2;
     uint64_t base = to_us_since_boot(t);
     uint64_t delayed = base + us;
-    if (delayed < base) {
-        delayed = (uint64_t)-1;
+    if ((int64_t)delayed < 0) {
+        // absolute_time_t (to allow for signed time deltas) is never greater than INT64_MAX which == at_the_end_of_time
+        delayed = INT64_MAX;
     }
     update_us_since_boot(&t2, delayed);
     return t2;
@@ -113,8 +114,9 @@ static inline absolute_time_t delayed_by_ms(const absolute_time_t t, uint32_t ms
     absolute_time_t t2;
     uint64_t base = to_us_since_boot(t);
     uint64_t delayed = base + ms * 1000ull;
-    if (delayed < base) {
-        delayed = (uint64_t)-1;
+    if ((int64_t)delayed < 0) {
+        // absolute_time_t (to allow for signed time deltas) is never greater than INT64_MAX which == at_the_end_of_time
+        delayed = INT64_MAX;
     }
     update_us_since_boot(&t2, delayed);
     return t2;
@@ -155,13 +157,34 @@ static inline int64_t absolute_time_diff_us(absolute_time_t from, absolute_time_
     return (int64_t)(to_us_since_boot(to) - to_us_since_boot(from));
 }
 
+/*! \brief Return the earlier of two timestamps
+ * \ingroup timestamp
+ *
+ * \param a the first timestamp
+ * \param b the second timestamp
+ * \return the earlier of the two timestamps
+ */
+static inline absolute_time_t absolute_time_min(absolute_time_t a, absolute_time_t b) {
+    return to_us_since_boot(a) < to_us_since_boot(b) ? a : b;
+}
+
 /*! \brief The timestamp representing the end of time; this is actually not the maximum possible
  * timestamp, but is set to 0x7fffffff_ffffffff microseconds to avoid sign overflows with time
- * arithmetic. This is still over 7 million years, so should be sufficient.
+ * arithmetic. This is almost 300,000 years, so should be sufficient.
  * \ingroup timestamp
  */
 extern const absolute_time_t at_the_end_of_time;
 
+/*! \brief Determine if the given timestamp is "at_the_end_of_time"
+ * \ingroup timestamp
+ *  \param t the timestamp
+ *  \return true if the timestamp is at_the_end_of_time
+ *  \sa at_the_end_of_time
+ */
+static inline bool is_at_the_end_of_time(absolute_time_t t) {
+    return to_us_since_boot(t) == to_us_since_boot(at_the_end_of_time);
+}
+
 /*! \brief The timestamp representing a null timestamp
  * \ingroup timestamp
  */
@@ -188,7 +211,7 @@ static inline bool is_nil_time(absolute_time_t t) {
  * \note  These functions should not be called from an IRQ handler.
  *
  * \note  Lower powered sleep requires use of the \link alarm_pool_get_default default alarm pool\endlink which may
- * be disabled by the #PICO_TIME_DEFAULT_ALARM_POOL_DISABLED define or currently full in which case these functions
+ * be disabled by the PICO_TIME_DEFAULT_ALARM_POOL_DISABLED #define or currently full in which case these functions
  * become busy waits instead.
  *
  * \note  Whilst \a sleep_ functions are preferable to \a busy_wait functions from a power perspective, the \a busy_wait equivalent function
@@ -385,6 +408,25 @@ alarm_pool_t *alarm_pool_get_default(void);
  */
 alarm_pool_t *alarm_pool_create(uint hardware_alarm_num, uint max_timers);
 
+/**
+ * \brief Create an alarm pool, claiming an used hardware alarm to back it.
+ *
+ * The alarm pool will call callbacks from an alarm IRQ Handler on the core of this function is called from.
+ *
+ * In many situations there is never any need for anything other than the default alarm pool, however you
+ * might want to create another if you want alarm callbacks on core 1 or require alarm pools of
+ * different priority (IRQ priority based preemption of callbacks)
+ *
+ * \note This method will hard assert if the there is no free hardware to claim.
+ *
+ * \ingroup alarm
+ * \param max_timers the maximum number of timers
+ *        \note For implementation reasons this is limited to PICO_PHEAP_MAX_ENTRIES which defaults to 255
+ * \sa alarm_pool_get_default()
+ * \sa hardware_claiming
+ */
+alarm_pool_t *alarm_pool_create_with_unused_hardware_alarm(uint max_timers);
+
 /**
  * \brief Return the hardware alarm used by an alarm pool
  * \ingroup alarm
@@ -393,6 +435,14 @@ alarm_pool_t *alarm_pool_create(uint hardware_alarm_num, uint max_timers);
  */
 uint alarm_pool_hardware_alarm_num(alarm_pool_t *pool);
 
+/**
+ * \brief Return the core number the alarm pool was initialized on (and hence callbacks are called on)
+ * \ingroup alarm
+ * \param pool the pool
+ * \return the core used by the pool
+ */
+uint alarm_pool_core_num(alarm_pool_t *pool);
+
 /**
  * \brief Destroy the alarm pool, cancelling all alarms and freeing up the underlying hardware alarm
  * \ingroup alarm
@@ -426,6 +476,25 @@ void alarm_pool_destroy(alarm_pool_t *pool);
  */
 alarm_id_t alarm_pool_add_alarm_at(alarm_pool_t *pool, absolute_time_t time, alarm_callback_t callback, void *user_data, bool fire_if_past);
 
+/*!
+ * \brief Add an alarm callback to be called at or after a specific time
+ * \ingroup alarm
+ *
+ * The callback is called as soon as possible after the time specified from an IRQ handler
+ * on the core the alarm pool was created on. Unlike \ref alarm_pool_add_alarm_at, this method
+ * guarantees to call the callback from that core even if the time is during this method call or in the past.
+ *
+ * \note It is safe to call this method from an IRQ handler (including alarm callbacks), and from either core.
+ *
+ * @param pool the alarm pool to use for scheduling the callback (this determines which hardware alarm is used, and which core calls the callback)
+ * @param time the timestamp when (after which) the callback should fire
+ * @param callback the callback function
+ * @param user_data user data to pass to the callback function
+ * @return >0 the alarm id for an active (at the time of return) alarm
+ * @return -1 if there were no alarm slots available
+ */
+alarm_id_t alarm_pool_add_alarm_at_force_in_context(alarm_pool_t *pool, absolute_time_t time, alarm_callback_t callback,
+                                                    void *user_data);
 /*!
  * \brief Add an alarm callback to be called after a delay specified in microseconds
  * \ingroup alarm
