@@ -6,6 +6,9 @@
 
 #include <stdio.h>
 #include <stdarg.h>
+#include <sys/time.h>
+#include <sys/times.h>
+#include <unistd.h>
 #include "pico.h"
 
 #include "hardware/regs/m0plus.h"
@@ -54,7 +57,7 @@ void runtime_install_stack_guard(void *stack_bottom) {
     // mask is 1 bit per 32 bytes of the 256 byte range... clear the bit for the segment we want
     uint32_t subregion_select = 0xffu ^ (1u << ((addr >> 5u) & 7u));
     mpu_hw->ctrl = 5; // enable mpu with background default map
-    mpu_hw->rbar = (addr & (uint)~0xff) | 0x8 | 0;
+    mpu_hw->rbar = (addr & (uint)~0xff) | M0PLUS_MPU_RBAR_VALID_BITS | 0;
     mpu_hw->rasr = 1 // enable region
                    | (0x7 << 1) // size 2^(7 + 1) = 256
                    | (subregion_select << 8)
@@ -175,7 +178,7 @@ void runtime_init(void) {
 
 }
 
-void _exit(__unused int status) {
+void __attribute__((noreturn)) __attribute__((weak)) _exit(__unused int status) {
 #if PICO_ENTER_USB_BOOT_ON_EXIT
     reset_usb_boot(0,0);
 #else
@@ -185,7 +188,7 @@ void _exit(__unused int status) {
 #endif
 }
 
-void *_sbrk(int incr) {
+__attribute__((weak)) void *_sbrk(int incr) {
     extern char end; /* Set by linker.  */
     static char *heap_end;
     char *prev_heap_end;
@@ -212,6 +215,45 @@ void *_sbrk(int incr) {
     return (void *) prev_heap_end;
 }
 
+static int64_t epoch_time_us_since_boot;
+
+__attribute__((weak)) int _gettimeofday (struct timeval *__restrict tv, __unused void *__restrict tz) {
+    if (tv) {
+        int64_t us_since_epoch = ((int64_t)to_us_since_boot(get_absolute_time())) - epoch_time_us_since_boot;
+        tv->tv_sec = (time_t)(us_since_epoch / 1000000);
+        tv->tv_usec = (suseconds_t)(us_since_epoch % 1000000);
+    }
+    return 0;
+}
+
+__attribute((weak)) int settimeofday(__unused const struct timeval *tv, __unused const struct timezone *tz) {
+    if (tv) {
+        int64_t us_since_epoch = tv->tv_sec * 1000000 + tv->tv_usec;
+        epoch_time_us_since_boot = (int64_t)to_us_since_boot(get_absolute_time()) - us_since_epoch;
+    }
+    return 0;
+}
+
+__attribute((weak)) int _times(struct tms *tms) {
+#if CLOCKS_PER_SEC >= 1000000
+    tms->tms_utime = (clock_t)(to_us_since_boot(get_absolute_time()) * (CLOCKS_PER_SEC / 1000000));
+#else
+    tms->tms_utime = (clock_t)(to_us_since_boot(get_absolute_time()) / (1000000 / CLOCKS_PER_SEC));
+#endif
+    tms->tms_stime = 0;
+    tms->tms_cutime = 0;
+    tms->tms_cstime = 0;
+    return 0;
+}
+
+__attribute((weak)) pid_t _getpid(void) {
+    return 0;
+}
+
+__attribute((weak)) int _kill(__unused pid_t pid, __unused int sig) {
+    return -1;
+}
+
 // exit is not useful... no desire to pull in __call_exitprocs
 void exit(int status) {
     _exit(status);
