@@ -1,8 +1,10 @@
 #define MICROPY_HW_BOARD_NAME               "ESP32S3 module (spiram)"
 #define MICROPY_HW_MCU_NAME                 "ESP32S3"
 
-#define MICROPY_PY_BLUETOOTH                (0)
 #define MICROPY_PY_MACHINE_DAC              (0)
 
+// Enable UART REPL for modules that have an external USB-UART and don't use native USB.
+#define MICROPY_HW_ENABLE_UART_REPL         (1)
+
 #define MICROPY_HW_I2C0_SCL                 (9)
 #define MICROPY_HW_I2C0_SDA                 (8)
