@@ -35,3 +35,17 @@
         .adc_list = (_adc_list), \
     } \
 
+#define PIN_LPSR(_name, _gpio, _pin, _af_list, _adc_list_len, _adc_list) \
+    { \
+        .base = { &machine_pin_type }, \
+        .name = MP_QSTR_##_name, \
+        .gpio = (_gpio), \
+        .pin = (uint32_t)(_pin), \
+        .muxRegister = (uint32_t)&(IOMUXC_LPSR->SW_MUX_CTL_PAD[kIOMUXC_LPSR_SW_MUX_CTL_PAD_##_name]), \
+        .configRegister = (uint32_t)&(IOMUXC_LPSR->SW_PAD_CTL_PAD[kIOMUXC_LPSR_SW_PAD_CTL_PAD_##_name]), \
+        .af_list_len = (uint8_t)(sizeof((_af_list)) / sizeof(machine_pin_af_obj_t)), \
+        .adc_list_len = (_adc_list_len), \
+        .af_list = (_af_list), \
+        .adc_list = (_adc_list), \
+    } \
+
