@@ -26,6 +26,7 @@
  */
 
 #include "py/runtime.h"
+#include "drivers/dht/dht.h"
 #include "extmod/machine_bitstream.h"
 #include "extmod/machine_mem.h"
 #include "extmod/machine_i2c.h"
@@ -36,8 +37,10 @@
 #include "led.h"
 #include "pin.h"
 #include "modmachine.h"
-#include "fsl_clock.h"
 #include "fsl_wdog.h"
+#if FSL_FEATURE_BOOT_ROM_HAS_ROMAPI
+#include "fsl_romapi.h"
+#endif
 
 #if MICROPY_PY_MACHINE
 
@@ -108,6 +111,24 @@ STATIC mp_obj_t machine_enable_irq(mp_obj_t state_in) {
 }
 MP_DEFINE_CONST_FUN_OBJ_1(machine_enable_irq_obj, machine_enable_irq);
 
+NORETURN mp_obj_t machine_bootloader(size_t n_args, const mp_obj_t *args) {
+    #if defined(MICROPY_BOARD_ENTER_BOOTLOADER)
+    // If a board has a custom bootloader, call it first.
+    MICROPY_BOARD_ENTER_BOOTLOADER(n_args, args);
+    #elif FSL_ROM_HAS_RUNBOOTLOADER_API
+    // If not, enter ROM bootloader in serial downloader / USB mode.
+    uint32_t arg = 0xEB110000;
+    ROM_RunBootloader(&arg);
+    #else
+    // No custom bootloader, or run bootloader API, then just reset.
+    WDOG_TriggerSystemSoftwareReset(WDOG1);
+    #endif
+    while (1) {
+        ;
+    }
+}
+STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(machine_bootloader_obj, 0, 1, machine_bootloader);
+
 STATIC const mp_rom_map_elem_t machine_module_globals_table[] = {
     { MP_ROM_QSTR(MP_QSTR___name__),            MP_ROM_QSTR(MP_QSTR_umachine) },
     { MP_ROM_QSTR(MP_QSTR_unique_id),           MP_ROM_PTR(&machine_unique_id_obj) },
@@ -144,11 +165,13 @@ STATIC const mp_rom_map_elem_t machine_module_globals_table[] = {
 
     { MP_ROM_QSTR(MP_QSTR_disable_irq),         MP_ROM_PTR(&machine_disable_irq_obj) },
     { MP_ROM_QSTR(MP_QSTR_enable_irq),          MP_ROM_PTR(&machine_enable_irq_obj) },
+    { MP_ROM_QSTR(MP_QSTR_bootloader),          MP_ROM_PTR(&machine_bootloader_obj) },
 
     #if MICROPY_PY_MACHINE_BITSTREAM
     { MP_ROM_QSTR(MP_QSTR_bitstream),           MP_ROM_PTR(&machine_bitstream_obj) },
     #endif
     { MP_ROM_QSTR(MP_QSTR_time_pulse_us),       MP_ROM_PTR(&machine_time_pulse_us_obj) },
+    { MP_ROM_QSTR(MP_QSTR_dht_readinto),        MP_ROM_PTR(&dht_readinto_obj) },
 
     // Reset reasons
     { MP_ROM_QSTR(MP_QSTR_PWRON_RESET),         MP_ROM_INT(MP_PWRON_RESET) },
