@@ -26,6 +26,7 @@
 
 #include "py/runtime.h"
 #include "py/mphal.h"
+#include "drivers/dht/dht.h"
 #include "shared/runtime/pyexec.h"
 #include "extmod/machine_bitstream.h"
 #include "extmod/machine_i2c.h"
@@ -38,10 +39,18 @@
 #include "modmachine.h"
 #include "uart.h"
 #include "hardware/clocks.h"
+#include "hardware/pll.h"
+#include "hardware/structs/rosc.h"
+#include "hardware/structs/scb.h"
+#include "hardware/structs/syscfg.h"
 #include "hardware/watchdog.h"
+#include "hardware/xosc.h"
 #include "pico/bootrom.h"
 #include "pico/stdlib.h"
 #include "pico/unique_id.h"
+#if MICROPY_PY_NETWORK_CYW43
+#include "lib/cyw43-driver/src/cyw43.h"
+#endif
 
 #if MICROPY_PY_MACHINE
 
@@ -81,11 +90,14 @@ STATIC mp_obj_t machine_reset_cause(void) {
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_0(machine_reset_cause_obj, machine_reset_cause);
 
-STATIC mp_obj_t machine_bootloader(void) {
+NORETURN mp_obj_t machine_bootloader(size_t n_args, const mp_obj_t *args) {
+    MICROPY_BOARD_ENTER_BOOTLOADER(n_args, args);
+    rosc_hw->ctrl = ROSC_CTRL_ENABLE_VALUE_ENABLE << ROSC_CTRL_ENABLE_LSB;
     reset_usb_boot(0, 0);
-    return mp_const_none;
+    for (;;) {
+    }
 }
-MP_DEFINE_CONST_FUN_OBJ_0(machine_bootloader_obj, machine_bootloader);
+STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(machine_bootloader_obj, 0, 1, machine_bootloader);
 
 STATIC mp_obj_t machine_freq(size_t n_args, const mp_obj_t *args) {
     if (n_args == 0) {
@@ -111,13 +123,88 @@ STATIC mp_obj_t machine_idle(void) {
 STATIC MP_DEFINE_CONST_FUN_OBJ_0(machine_idle_obj, machine_idle);
 
 STATIC mp_obj_t machine_lightsleep(size_t n_args, const mp_obj_t *args) {
-    if (n_args == 0) {
-        for (;;) {
-            MICROPY_EVENT_POLL_HOOK
+    mp_int_t delay_ms = 0;
+    bool use_timer_alarm = false;
+
+    if (n_args == 1) {
+        delay_ms = mp_obj_get_int(args[0]);
+        if (delay_ms <= 1) {
+            // Sleep is too small, just use standard delay.
+            mp_hal_delay_ms(delay_ms);
+            return mp_const_none;
+        }
+        use_timer_alarm = delay_ms < (1ULL << 32) / 1000;
+        if (use_timer_alarm) {
+            // Use timer alarm to wake.
+        } else {
+            // TODO: Use RTC alarm to wake.
+            mp_raise_ValueError(MP_ERROR_TEXT("sleep too long"));
         }
-    } else {
-        mp_hal_delay_ms(mp_obj_get_int(args[0]));
     }
+
+    const uint32_t xosc_hz = XOSC_MHZ * 1000000;
+
+    uint32_t my_interrupts = save_and_disable_interrupts();
+    #if MICROPY_PY_NETWORK_CYW43
+    if (cyw43_has_pending) {
+        restore_interrupts(my_interrupts);
+        return mp_const_none;
+    }
+    #endif
+    // Disable USB and ADC clocks.
+    clock_stop(clk_usb);
+    clock_stop(clk_adc);
+
+    // CLK_REF = XOSC
+    clock_configure(clk_ref, CLOCKS_CLK_REF_CTRL_SRC_VALUE_XOSC_CLKSRC, 0, xosc_hz, xosc_hz);
+
+    // CLK_SYS = CLK_REF
+    clock_configure(clk_sys, CLOCKS_CLK_SYS_CTRL_SRC_VALUE_CLK_REF, 0, xosc_hz, xosc_hz);
+
+    // CLK_RTC = XOSC / 256
+    clock_configure(clk_rtc, 0, CLOCKS_CLK_RTC_CTRL_AUXSRC_VALUE_XOSC_CLKSRC, xosc_hz, xosc_hz / 256);
+
+    // CLK_PERI = CLK_SYS
+    clock_configure(clk_peri, 0, CLOCKS_CLK_PERI_CTRL_AUXSRC_VALUE_CLK_SYS, xosc_hz, xosc_hz);
+
+    // Disable PLLs.
+    pll_deinit(pll_sys);
+    pll_deinit(pll_usb);
+
+    // Disable ROSC.
+    rosc_hw->ctrl = ROSC_CTRL_ENABLE_VALUE_DISABLE << ROSC_CTRL_ENABLE_LSB;
+
+    if (n_args == 0) {
+        #if MICROPY_PY_NETWORK_CYW43
+        gpio_set_dormant_irq_enabled(CYW43_PIN_WL_HOST_WAKE, GPIO_IRQ_LEVEL_HIGH, true);
+        #endif
+        xosc_dormant();
+    } else {
+        uint32_t sleep_en0 = clocks_hw->sleep_en0;
+        uint32_t sleep_en1 = clocks_hw->sleep_en1;
+        clocks_hw->sleep_en0 = CLOCKS_SLEEP_EN0_CLK_RTC_RTC_BITS;
+        if (use_timer_alarm) {
+            // Use timer alarm to wake.
+            clocks_hw->sleep_en1 = CLOCKS_SLEEP_EN1_CLK_SYS_TIMER_BITS;
+            timer_hw->alarm[3] = timer_hw->timerawl + delay_ms * 1000;
+        } else {
+            // TODO: Use RTC alarm to wake.
+            clocks_hw->sleep_en1 = 0;
+        }
+        scb_hw->scr |= M0PLUS_SCR_SLEEPDEEP_BITS;
+        __wfi();
+        scb_hw->scr &= ~M0PLUS_SCR_SLEEPDEEP_BITS;
+        clocks_hw->sleep_en0 = sleep_en0;
+        clocks_hw->sleep_en1 = sleep_en1;
+    }
+
+    // Enable ROSC.
+    rosc_hw->ctrl = ROSC_CTRL_ENABLE_VALUE_ENABLE << ROSC_CTRL_ENABLE_LSB;
+
+    // Bring back all clocks.
+    clocks_init();
+    restore_interrupts(my_interrupts);
+
     return mp_const_none;
 }
 MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(machine_lightsleep_obj, 0, 1, machine_lightsleep);
@@ -161,13 +248,14 @@ STATIC const mp_rom_map_elem_t machine_module_globals_table[] = {
     { MP_ROM_QSTR(MP_QSTR_bitstream),           MP_ROM_PTR(&machine_bitstream_obj) },
     #endif
     { MP_ROM_QSTR(MP_QSTR_time_pulse_us),       MP_ROM_PTR(&machine_time_pulse_us_obj) },
+    { MP_ROM_QSTR(MP_QSTR_dht_readinto),        MP_ROM_PTR(&dht_readinto_obj) },
 
     { MP_ROM_QSTR(MP_QSTR_mem8),                MP_ROM_PTR(&machine_mem8_obj) },
     { MP_ROM_QSTR(MP_QSTR_mem16),               MP_ROM_PTR(&machine_mem16_obj) },
     { MP_ROM_QSTR(MP_QSTR_mem32),               MP_ROM_PTR(&machine_mem32_obj) },
 
     { MP_ROM_QSTR(MP_QSTR_ADC),                 MP_ROM_PTR(&machine_adc_type) },
-    { MP_ROM_QSTR(MP_QSTR_I2C),                 MP_ROM_PTR(&machine_hw_i2c_type) },
+    { MP_ROM_QSTR(MP_QSTR_I2C),                 MP_ROM_PTR(&machine_i2c_type) },
     { MP_ROM_QSTR(MP_QSTR_SoftI2C),             MP_ROM_PTR(&mp_machine_soft_i2c_type) },
     { MP_ROM_QSTR(MP_QSTR_I2S),                 MP_ROM_PTR(&machine_i2s_type) },
     { MP_ROM_QSTR(MP_QSTR_Pin),                 MP_ROM_PTR(&machine_pin_type) },
