@@ -27,18 +27,20 @@
 #include "py/runtime.h"
 #include "extmod/utime_mphal.h"
 #include "shared/timeutils/timeutils.h"
+#include "modmachine.h"
 
 // localtime([secs])
 STATIC mp_obj_t time_localtime(size_t n_args, const mp_obj_t *args) {
     timeutils_struct_time_t tm;
     mp_int_t seconds;
+
     if (n_args == 0 || args[0] == mp_const_none) {
-        // seconds = pyb_rtc_get_us_since_epoch() / 1000 / 1000;
-        seconds = mp_obj_get_int(args[0]);
+        rtc_gettime(&tm);
     } else {
         seconds = mp_obj_get_int(args[0]);
+        timeutils_seconds_since_epoch_to_struct_time(seconds, &tm);
     }
-    timeutils_seconds_since_epoch_to_struct_time(seconds, &tm);
+
     mp_obj_t tuple[8] = {
         tuple[0] = mp_obj_new_int(tm.tm_year),
         tuple[1] = mp_obj_new_int(tm.tm_mon),
@@ -46,8 +48,8 @@ STATIC mp_obj_t time_localtime(size_t n_args, const mp_obj_t *args) {
         tuple[3] = mp_obj_new_int(tm.tm_hour),
         tuple[4] = mp_obj_new_int(tm.tm_min),
         tuple[5] = mp_obj_new_int(tm.tm_sec),
-        tuple[6] = mp_obj_new_int(tm.tm_wday),
-        tuple[7] = mp_obj_new_int(tm.tm_yday),
+        tuple[6] = mp_obj_new_int(timeutils_calc_weekday(tm.tm_year, tm.tm_mon, tm.tm_mday)),
+        tuple[7] = mp_obj_new_int(timeutils_year_day(tm.tm_year, tm.tm_mon, tm.tm_mday)),
     };
     return mp_obj_new_tuple(8, tuple);
 }
@@ -72,7 +74,10 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(time_mktime_obj, time_mktime);
 
 // time()
 STATIC mp_obj_t time_time(void) {
-    mp_raise_NotImplementedError("time");
+    timeutils_struct_time_t tm;
+    rtc_gettime(&tm);
+    return mp_obj_new_int_from_uint(timeutils_mktime(
+        tm.tm_year, tm.tm_mon, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec));
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_0(time_time_obj, time_time);
 
