@@ -261,6 +261,8 @@ STATIC mp_obj_t pyb_dac_init_helper(pyb_dac_obj_t *self, size_t n_args, const mp
     __HAL_RCC_DAC12_CLK_ENABLE();
     #elif defined(STM32F0) || defined(STM32G0) || defined(STM32G4) || defined(STM32L4)
     __HAL_RCC_DAC1_CLK_ENABLE();
+    #elif defined(STM32L1)
+    __HAL_RCC_DAC_CLK_ENABLE();
     #else
     #error Unsupported Processor
     #endif
@@ -502,12 +504,13 @@ STATIC const mp_rom_map_elem_t pyb_dac_locals_dict_table[] = {
 
 STATIC MP_DEFINE_CONST_DICT(pyb_dac_locals_dict, pyb_dac_locals_dict_table);
 
-const mp_obj_type_t pyb_dac_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_DAC,
-    .print = pyb_dac_print,
-    .make_new = pyb_dac_make_new,
-    .locals_dict = (mp_obj_dict_t *)&pyb_dac_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    pyb_dac_type,
+    MP_QSTR_DAC,
+    MP_TYPE_FLAG_NONE,
+    make_new, pyb_dac_make_new,
+    print, pyb_dac_print,
+    locals_dict, &pyb_dac_locals_dict
+    );
 
 #endif // MICROPY_HW_ENABLE_DAC
