@@ -24,10 +24,6 @@
  * THE SOFTWARE.
  */
 
-#include <stdio.h>
-#include <stddef.h>
-#include <string.h>
-
 #include "py/runtime.h"
 #include "py/gc.h"
 #include "py/mphal.h"
@@ -36,6 +32,11 @@
 #include "extint.h"
 #include "irq.h"
 
+#if MICROPY_PY_NETWORK_CYW43 && defined(pyb_pin_WL_HOST_WAKE)
+#include "lib/cyw43-driver/src/cyw43.h"
+#include "lib/cyw43-driver/src/cyw43_stats.h"
+#endif
+
 /// \moduleref pyb
 /// \class ExtInt - configure I/O pins to interrupt on external events
 ///
@@ -91,7 +92,7 @@
 #define EXTI_SWIER_BB(line) (*(__IO uint32_t *)(PERIPH_BB_BASE + ((EXTI_OFFSET + offsetof(EXTI_TypeDef, SWIER)) * 32) + ((line) * 4)))
 #endif
 
-#if defined(STM32L4) || defined(STM32WB)
+#if defined(STM32G0) || defined(STM32G4) || defined(STM32L4) || defined(STM32WB) || defined(STM32WL)
 // The L4 MCU supports 40 Events/IRQs lines of the type configurable and direct.
 // Here we only support configurable line types.  Details, see page 330 of RM0351, Rev 1.
 // The USB_FS_WAKUP event is a direct type and there is no support for it.
@@ -138,8 +139,12 @@ STATIC mp_obj_t pyb_extint_callback_arg[EXTI_NUM_VECTORS];
 #define OTG_FS_WKUP_IRQn 42  // Some MCUs don't have FS IRQ, but we want a value to put in our table
 #endif
 
+#if defined(STM32G4)
+#define TAMP_STAMP_IRQn RTC_TAMP_LSECSS_IRQn
+#endif
+
 STATIC const uint8_t nvic_irq_channel[EXTI_NUM_VECTORS] = {
-    #if defined(STM32F0) || defined(STM32L0)
+    #if defined(STM32F0) || defined(STM32L0) || defined(STM32G0)
 
     EXTI0_1_IRQn,  EXTI0_1_IRQn,  EXTI2_3_IRQn,  EXTI2_3_IRQn,
     EXTI4_15_IRQn, EXTI4_15_IRQn, EXTI4_15_IRQn, EXTI4_15_IRQn,
@@ -150,12 +155,20 @@ STATIC const uint8_t nvic_irq_channel[EXTI_NUM_VECTORS] = {
     #else
     PVD_VDDIO2_IRQn,
     #endif
+    #if defined(STM32G0)
+    ADC1_COMP_IRQn,
+    ADC1_COMP_IRQn,
+    RTC_TAMP_IRQn,
+    0, // COMP3
+    RTC_TAMP_IRQn,// 21
+    #else
     RTC_IRQn,
     0, // internal USB wakeup event
     RTC_IRQn,
     RTC_IRQn,
     ADC1_COMP_IRQn,
     ADC1_COMP_IRQn,
+    #endif
 
     #else
 
@@ -177,8 +190,14 @@ STATIC const uint8_t nvic_irq_channel[EXTI_NUM_VECTORS] = {
     RTC_Alarm_IRQn,
     TAMP_STAMP_LSECSS_IRQn,
     RTC_WKUP_IRQn,
+    #elif defined(STM32WL)
+    PVD_PVM_IRQn,
+    RTC_Alarm_IRQn,
+    TAMP_STAMP_LSECSS_SSRU_IRQn, // SSRU
+    TAMP_STAMP_LSECSS_SSRU_IRQn, // TAMP, RTC_STAMP, LSE_CSS
+    RTC_WKUP_IRQn,
     #else
-    #if defined(STM32L4)
+    #if defined(STM32G4) || defined(STM32L4)
     PVD_PVM_IRQn,
     #else
     PVD_IRQn,
@@ -192,7 +211,11 @@ STATIC const uint8_t nvic_irq_channel[EXTI_NUM_VECTORS] = {
     #endif
     ETH_WKUP_IRQn,
     OTG_HS_WKUP_IRQn,
+    #if defined(STM32L1)
+    TAMPER_STAMP_IRQn,
+    #else
     TAMP_STAMP_IRQn,
+    #endif
     RTC_WKUP_IRQn,
     #endif
 
@@ -304,12 +327,18 @@ void extint_register_pin(const pin_obj_t *pin, uint32_t mode, bool hard_irq, mp_
         pyb_extint_callback_arg[line] = MP_OBJ_FROM_PTR(pin);
 
         // Route the GPIO to EXTI
-        #if !defined(STM32WB)
+        #if !defined(STM32WB) && !defined(STM32WL)
         __HAL_RCC_SYSCFG_CLK_ENABLE();
         #endif
+        #if defined(STM32G0)
+        EXTI->EXTICR[line >> 2] =
+            (EXTI->EXTICR[line >> 2] & ~(0x0f << (4 * (line & 0x03))))
+            | ((uint32_t)(GPIO_GET_INDEX(pin->gpio)) << (4 * (line & 0x03)));
+        #else
         SYSCFG->EXTICR[line >> 2] =
             (SYSCFG->EXTICR[line >> 2] & ~(0x0f << (4 * (line & 0x03))))
             | ((uint32_t)(GPIO_GET_INDEX(pin->gpio)) << (4 * (line & 0x03)));
+        #endif
 
         extint_trigger_mode(line, mode);
 
@@ -341,12 +370,18 @@ void extint_set(const pin_obj_t *pin, uint32_t mode) {
         pyb_extint_callback_arg[line] = MP_OBJ_FROM_PTR(pin);
 
         // Route the GPIO to EXTI
-        #if !defined(STM32WB)
+        #if !defined(STM32WB) && !defined(STM32WL)
         __HAL_RCC_SYSCFG_CLK_ENABLE();
         #endif
+        #if defined(STM32G0)
+        EXTI->EXTICR[line >> 2] =
+            (EXTI->EXTICR[line >> 2] & ~(0x0f << (4 * (line & 0x03))))
+            | ((uint32_t)(GPIO_GET_INDEX(pin->gpio)) << (4 * (line & 0x03)));
+        #else
         SYSCFG->EXTICR[line >> 2] =
             (SYSCFG->EXTICR[line >> 2] & ~(0x0f << (4 * (line & 0x03))))
             | ((uint32_t)(GPIO_GET_INDEX(pin->gpio)) << (4 * (line & 0x03)));
+        #endif
 
         // Enable or disable the rising detector
         if ((mode & GPIO_MODE_IT_RISING) == GPIO_MODE_IT_RISING) {
@@ -381,7 +416,7 @@ void extint_enable(uint line) {
     if (pyb_extint_mode[line] == EXTI_Mode_Interrupt) {
         #if defined(STM32H7)
         EXTI_D1->IMR1 |= (1 << line);
-        #elif defined(STM32WB)
+        #elif defined(STM32G0) || defined(STM32G4) || defined(STM32WB) || defined(STM32WL)
         EXTI->IMR1 |= (1 << line);
         #else
         EXTI->IMR |= (1 << line);
@@ -389,7 +424,7 @@ void extint_enable(uint line) {
     } else {
         #if defined(STM32H7)
         EXTI_D1->EMR1 |= (1 << line);
-        #elif defined(STM32WB)
+        #elif defined(STM32G0) || defined(STM32G4) || defined(STM32WB) || defined(STM32WL)
         EXTI->EMR1 |= (1 << line);
         #else
         EXTI->EMR |= (1 << line);
@@ -415,7 +450,7 @@ void extint_disable(uint line) {
     #if defined(STM32H7)
     EXTI_D1->IMR1 &= ~(1 << line);
     EXTI_D1->EMR1 &= ~(1 << line);
-    #elif defined(STM32WB)
+    #elif defined(STM32G0) || defined(STM32G4) || defined(STM32WB) || defined(STM32WL)
     EXTI->IMR1 &= ~(1 << line);
     EXTI->EMR1 &= ~(1 << line);
     #else
@@ -437,7 +472,7 @@ void extint_swint(uint line) {
         return;
     }
     // we need 0 to 1 transition to trigger the interrupt
-    #if defined(STM32L4) || defined(STM32H7) || defined(STM32WB)
+    #if defined(STM32G0) || defined(STM32G4) || defined(STM32H7) || defined(STM32L4) || defined(STM32WB) || defined(STM32WL)
     EXTI->SWIER1 &= ~(1 << line);
     EXTI->SWIER1 |= (1 << line);
     #else
@@ -515,46 +550,56 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(extint_obj_swint_obj,  extint_obj_swint);
 /// \classmethod regs()
 /// Dump the values of the EXTI registers.
 STATIC mp_obj_t extint_regs(void) {
-    #if defined(STM32L4) || defined(STM32WB)
-    printf("EXTI_IMR1   %08x\n", (unsigned int)EXTI->IMR1);
-    printf("EXTI_IMR2   %08x\n", (unsigned int)EXTI->IMR2);
-    printf("EXTI_EMR1   %08x\n", (unsigned int)EXTI->EMR1);
-    printf("EXTI_EMR2   %08x\n", (unsigned int)EXTI->EMR2);
-    printf("EXTI_RTSR1  %08x\n", (unsigned int)EXTI->RTSR1);
-    printf("EXTI_RTSR2  %08x\n", (unsigned int)EXTI->RTSR2);
-    printf("EXTI_FTSR1  %08x\n", (unsigned int)EXTI->FTSR1);
-    printf("EXTI_FTSR2  %08x\n", (unsigned int)EXTI->FTSR2);
-    printf("EXTI_SWIER1 %08x\n", (unsigned int)EXTI->SWIER1);
-    printf("EXTI_SWIER2 %08x\n", (unsigned int)EXTI->SWIER2);
-    printf("EXTI_PR1    %08x\n", (unsigned int)EXTI->PR1);
-    printf("EXTI_PR2    %08x\n", (unsigned int)EXTI->PR2);
-    #elif defined(STM32H7)
-    printf("EXTI_IMR1   %08x\n", (unsigned int)EXTI_D1->IMR1);
-    printf("EXTI_IMR2   %08x\n", (unsigned int)EXTI_D1->IMR2);
-    printf("EXTI_IMR3   %08x\n", (unsigned int)EXTI_D1->IMR3);
-    printf("EXTI_EMR1   %08x\n", (unsigned int)EXTI_D1->EMR1);
-    printf("EXTI_EMR2   %08x\n", (unsigned int)EXTI_D1->EMR2);
-    printf("EXTI_EMR3   %08x\n", (unsigned int)EXTI_D1->EMR3);
-    printf("EXTI_RTSR1  %08x\n", (unsigned int)EXTI->RTSR1);
-    printf("EXTI_RTSR2  %08x\n", (unsigned int)EXTI->RTSR2);
-    printf("EXTI_RTSR3  %08x\n", (unsigned int)EXTI->RTSR3);
-    printf("EXTI_FTSR1  %08x\n", (unsigned int)EXTI->FTSR1);
-    printf("EXTI_FTSR2  %08x\n", (unsigned int)EXTI->FTSR2);
-    printf("EXTI_FTSR3  %08x\n", (unsigned int)EXTI->FTSR3);
-    printf("EXTI_SWIER1 %08x\n", (unsigned int)EXTI->SWIER1);
-    printf("EXTI_SWIER2 %08x\n", (unsigned int)EXTI->SWIER2);
-    printf("EXTI_SWIER3 %08x\n", (unsigned int)EXTI->SWIER3);
-    printf("EXTI_PR1    %08x\n", (unsigned int)EXTI_D1->PR1);
-    printf("EXTI_PR2    %08x\n", (unsigned int)EXTI_D1->PR2);
-    printf("EXTI_PR3    %08x\n", (unsigned int)EXTI_D1->PR3);
+    const mp_print_t *print = &mp_plat_print;
+
+    #if defined(STM32G0) || defined(STM32G4) || defined(STM32L4) || defined(STM32WB) || defined(STM32WL)
+    mp_printf(print, "EXTI_IMR1   %08x\n", (unsigned int)EXTI->IMR1);
+    mp_printf(print, "EXTI_IMR2   %08x\n", (unsigned int)EXTI->IMR2);
+    mp_printf(print, "EXTI_EMR1   %08x\n", (unsigned int)EXTI->EMR1);
+    mp_printf(print, "EXTI_EMR2   %08x\n", (unsigned int)EXTI->EMR2);
+    mp_printf(print, "EXTI_RTSR1  %08x\n", (unsigned int)EXTI->RTSR1);
+    mp_printf(print, "EXTI_RTSR2  %08x\n", (unsigned int)EXTI->RTSR2);
+    mp_printf(print, "EXTI_FTSR1  %08x\n", (unsigned int)EXTI->FTSR1);
+    mp_printf(print, "EXTI_FTSR2  %08x\n", (unsigned int)EXTI->FTSR2);
+    mp_printf(print, "EXTI_SWIER1 %08x\n", (unsigned int)EXTI->SWIER1);
+    mp_printf(print, "EXTI_SWIER2 %08x\n", (unsigned int)EXTI->SWIER2);
+    #if defined(STM32G0)
+    mp_printf(print, "EXTI_RPR1    %08x\n", (unsigned int)EXTI->RPR1);
+    mp_printf(print, "EXTI_FPR1    %08x\n", (unsigned int)EXTI->FPR1);
+    mp_printf(print, "EXTI_RPR2    %08x\n", (unsigned int)EXTI->RPR2);
+    mp_printf(print, "EXTI_FPR2    %08x\n", (unsigned int)EXTI->FPR2);
     #else
-    printf("EXTI_IMR   %08x\n", (unsigned int)EXTI->IMR);
-    printf("EXTI_EMR   %08x\n", (unsigned int)EXTI->EMR);
-    printf("EXTI_RTSR  %08x\n", (unsigned int)EXTI->RTSR);
-    printf("EXTI_FTSR  %08x\n", (unsigned int)EXTI->FTSR);
-    printf("EXTI_SWIER %08x\n", (unsigned int)EXTI->SWIER);
-    printf("EXTI_PR    %08x\n", (unsigned int)EXTI->PR);
+    mp_printf(print, "EXTI_PR1    %08x\n", (unsigned int)EXTI->PR1);
+    mp_printf(print, "EXTI_PR2    %08x\n", (unsigned int)EXTI->PR2);
     #endif
+    #elif defined(STM32H7)
+    mp_printf(print, "EXTI_IMR1   %08x\n", (unsigned int)EXTI_D1->IMR1);
+    mp_printf(print, "EXTI_IMR2   %08x\n", (unsigned int)EXTI_D1->IMR2);
+    mp_printf(print, "EXTI_IMR3   %08x\n", (unsigned int)EXTI_D1->IMR3);
+    mp_printf(print, "EXTI_EMR1   %08x\n", (unsigned int)EXTI_D1->EMR1);
+    mp_printf(print, "EXTI_EMR2   %08x\n", (unsigned int)EXTI_D1->EMR2);
+    mp_printf(print, "EXTI_EMR3   %08x\n", (unsigned int)EXTI_D1->EMR3);
+    mp_printf(print, "EXTI_RTSR1  %08x\n", (unsigned int)EXTI->RTSR1);
+    mp_printf(print, "EXTI_RTSR2  %08x\n", (unsigned int)EXTI->RTSR2);
+    mp_printf(print, "EXTI_RTSR3  %08x\n", (unsigned int)EXTI->RTSR3);
+    mp_printf(print, "EXTI_FTSR1  %08x\n", (unsigned int)EXTI->FTSR1);
+    mp_printf(print, "EXTI_FTSR2  %08x\n", (unsigned int)EXTI->FTSR2);
+    mp_printf(print, "EXTI_FTSR3  %08x\n", (unsigned int)EXTI->FTSR3);
+    mp_printf(print, "EXTI_SWIER1 %08x\n", (unsigned int)EXTI->SWIER1);
+    mp_printf(print, "EXTI_SWIER2 %08x\n", (unsigned int)EXTI->SWIER2);
+    mp_printf(print, "EXTI_SWIER3 %08x\n", (unsigned int)EXTI->SWIER3);
+    mp_printf(print, "EXTI_PR1    %08x\n", (unsigned int)EXTI_D1->PR1);
+    mp_printf(print, "EXTI_PR2    %08x\n", (unsigned int)EXTI_D1->PR2);
+    mp_printf(print, "EXTI_PR3    %08x\n", (unsigned int)EXTI_D1->PR3);
+    #else
+    mp_printf(print, "EXTI_IMR   %08x\n", (unsigned int)EXTI->IMR);
+    mp_printf(print, "EXTI_EMR   %08x\n", (unsigned int)EXTI->EMR);
+    mp_printf(print, "EXTI_RTSR  %08x\n", (unsigned int)EXTI->RTSR);
+    mp_printf(print, "EXTI_FTSR  %08x\n", (unsigned int)EXTI->FTSR);
+    mp_printf(print, "EXTI_SWIER %08x\n", (unsigned int)EXTI->SWIER);
+    mp_printf(print, "EXTI_PR    %08x\n", (unsigned int)EXTI->PR);
+    #endif
+
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_0(extint_regs_fun_obj, extint_regs);
@@ -590,8 +635,7 @@ STATIC mp_obj_t extint_make_new(const mp_obj_type_t *type, size_t n_args, size_t
     mp_arg_val_t vals[PYB_EXTINT_MAKE_NEW_NUM_ARGS];
     mp_arg_parse_all_kw_array(n_args, n_kw, args, PYB_EXTINT_MAKE_NEW_NUM_ARGS, pyb_extint_make_new_args, vals);
 
-    extint_obj_t *self = m_new_obj(extint_obj_t);
-    self->base.type = type;
+    extint_obj_t *self = mp_obj_malloc(extint_obj_t, type);
     self->line = extint_register(vals[0].u_obj, vals[1].u_int, vals[2].u_int, vals[3].u_obj, false);
 
     return MP_OBJ_FROM_PTR(self);
@@ -623,13 +667,14 @@ STATIC const mp_rom_map_elem_t extint_locals_dict_table[] = {
 
 STATIC MP_DEFINE_CONST_DICT(extint_locals_dict, extint_locals_dict_table);
 
-const mp_obj_type_t extint_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_ExtInt,
-    .print = extint_obj_print,
-    .make_new = extint_make_new,
-    .locals_dict = (mp_obj_dict_t *)&extint_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    extint_type,
+    MP_QSTR_ExtInt,
+    MP_TYPE_FLAG_NONE,
+    make_new, extint_make_new,
+    print, extint_obj_print,
+    locals_dict, &extint_locals_dict
+    );
 
 void extint_init0(void) {
     for (int i = 0; i < PYB_EXTI_NUM_VECTORS; i++) {
@@ -649,9 +694,9 @@ void Handle_EXTI_Irq(uint32_t line) {
             mp_obj_t *cb = &MP_STATE_PORT(pyb_extint_callback)[line];
             #if MICROPY_PY_NETWORK_CYW43 && defined(pyb_pin_WL_HOST_WAKE)
             if (pyb_extint_callback_arg[line] == MP_OBJ_FROM_PTR(pyb_pin_WL_HOST_WAKE)) {
-                extern void (*cyw43_poll)(void);
                 if (cyw43_poll) {
                     pendsv_schedule_dispatch(PENDSV_DISPATCH_CYW43, cyw43_poll);
+                    CYW43_STAT_INC(IRQ_COUNT);
                 }
                 return;
             }
@@ -684,3 +729,5 @@ void Handle_EXTI_Irq(uint32_t line) {
         }
     }
 }
+
+MP_REGISTER_ROOT_POINTER(mp_obj_t pyb_extint_callback[PYB_EXTI_NUM_VECTORS]);
