@@ -24,10 +24,6 @@
  * THE SOFTWARE.
  */
 
-#include <stdio.h>
-#include <stdint.h>
-#include <string.h>
-
 #include "py/runtime.h"
 #include "py/mphal.h"
 #include "extmod/virtpin.h"
@@ -102,15 +98,17 @@ void pin_init0(void) {
 
 // C API used to convert a user-supplied pin name into an ordinal pin number.
 const pin_obj_t *pin_find(mp_obj_t user_obj) {
+    const mp_print_t *print = &mp_plat_print;
+
     const pin_obj_t *pin_obj;
 
     // If a pin was provided, then use it
     if (mp_obj_is_type(user_obj, &pin_type)) {
         pin_obj = MP_OBJ_TO_PTR(user_obj);
         if (pin_class_debug) {
-            printf("Pin map passed pin ");
+            mp_printf(print, "Pin map passed pin ");
             mp_obj_print(MP_OBJ_FROM_PTR(pin_obj), PRINT_STR);
-            printf("\n");
+            mp_printf(print, "\n");
         }
         return pin_obj;
     }
@@ -122,11 +120,11 @@ const pin_obj_t *pin_find(mp_obj_t user_obj) {
                 mp_raise_ValueError(MP_ERROR_TEXT("Pin.mapper didn't return a Pin object"));
             }
             if (pin_class_debug) {
-                printf("Pin.mapper maps ");
+                mp_printf(print, "Pin.mapper maps ");
                 mp_obj_print(user_obj, PRINT_REPR);
-                printf(" to ");
+                mp_printf(print, " to ");
                 mp_obj_print(o, PRINT_STR);
-                printf("\n");
+                mp_printf(print, "\n");
             }
             return MP_OBJ_TO_PTR(o);
         }
@@ -140,11 +138,11 @@ const pin_obj_t *pin_find(mp_obj_t user_obj) {
         if (elem != NULL && elem->value != MP_OBJ_NULL) {
             mp_obj_t o = elem->value;
             if (pin_class_debug) {
-                printf("Pin.map_dict maps ");
+                mp_printf(print, "Pin.map_dict maps ");
                 mp_obj_print(user_obj, PRINT_REPR);
-                printf(" to ");
+                mp_printf(print, " to ");
                 mp_obj_print(o, PRINT_STR);
-                printf("\n");
+                mp_printf(print, "\n");
             }
             return MP_OBJ_TO_PTR(o);
         }
@@ -154,11 +152,11 @@ const pin_obj_t *pin_find(mp_obj_t user_obj) {
     pin_obj = pin_find_named_pin(&pin_board_pins_locals_dict, user_obj);
     if (pin_obj) {
         if (pin_class_debug) {
-            printf("Pin.board maps ");
+            mp_printf(print, "Pin.board maps ");
             mp_obj_print(user_obj, PRINT_REPR);
-            printf(" to ");
+            mp_printf(print, " to ");
             mp_obj_print(MP_OBJ_FROM_PTR(pin_obj), PRINT_STR);
-            printf("\n");
+            mp_printf(print, "\n");
         }
         return pin_obj;
     }
@@ -167,11 +165,11 @@ const pin_obj_t *pin_find(mp_obj_t user_obj) {
     pin_obj = pin_find_named_pin(&pin_cpu_pins_locals_dict, user_obj);
     if (pin_obj) {
         if (pin_class_debug) {
-            printf("Pin.cpu maps ");
+            mp_printf(print, "Pin.cpu maps ");
             mp_obj_print(user_obj, PRINT_REPR);
-            printf(" to ");
+            mp_printf(print, " to ");
             mp_obj_print(MP_OBJ_FROM_PTR(pin_obj), PRINT_STR);
-            printf("\n");
+            mp_printf(print, "\n");
         }
         return pin_obj;
     }
@@ -230,9 +228,9 @@ STATIC void pin_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t
             mp_uint_t af_idx = pin_get_af(self);
             const pin_af_obj_t *af_obj = pin_find_af_by_index(self, af_idx);
             if (af_obj == NULL) {
-                mp_printf(print, ", af=%d)", af_idx);
+                mp_printf(print, ", alt=%d)", af_idx);
             } else {
-                mp_printf(print, ", af=Pin.%q)", af_obj->name);
+                mp_printf(print, ", alt=Pin.%q)", af_obj->name);
             }
         } else {
             mp_print_str(print, ")");
@@ -328,7 +326,7 @@ STATIC mp_obj_t pin_debug(size_t n_args, const mp_obj_t *args) {
 STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(pin_debug_fun_obj, 1, 2, pin_debug);
 STATIC MP_DEFINE_CONST_CLASSMETHOD_OBJ(pin_debug_obj, MP_ROM_PTR(&pin_debug_fun_obj));
 
-// init(mode, pull=None, af=-1, *, value, alt)
+// init(mode, pull=None, alt=-1, *, value, alt)
 STATIC mp_obj_t pin_obj_init_helper(const pin_obj_t *self, size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     static const mp_arg_t allowed_args[] = {
         { MP_QSTR_mode, MP_ARG_REQUIRED | MP_ARG_INT },
@@ -592,15 +590,16 @@ STATIC const mp_pin_p_t pin_pin_p = {
     .ioctl = pin_ioctl,
 };
 
-const mp_obj_type_t pin_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_Pin,
-    .print = pin_print,
-    .make_new = mp_pin_make_new,
-    .call = pin_call,
-    .protocol = &pin_pin_p,
-    .locals_dict = (mp_obj_dict_t *)&pin_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    pin_type,
+    MP_QSTR_Pin,
+    MP_TYPE_FLAG_NONE,
+    make_new, mp_pin_make_new,
+    print, pin_print,
+    call, pin_call,
+    protocol, &pin_pin_p,
+    locals_dict, &pin_locals_dict
+    );
 
 /// \moduleref pyb
 /// \class PinAF - Pin Alternate Functions
@@ -625,9 +624,9 @@ const mp_obj_type_t pin_type = {
 /// is desired.
 ///
 /// To configure X3 to expose TIM2_CH3, you could use:
-///    pin = pyb.Pin(pyb.Pin.board.X3, mode=pyb.Pin.AF_PP, af=pyb.Pin.AF1_TIM2)
+///    pin = pyb.Pin(pyb.Pin.board.X3, mode=pyb.Pin.AF_PP, alt=pyb.Pin.AF1_TIM2)
 /// or:
-///    pin = pyb.Pin(pyb.Pin.board.X3, mode=pyb.Pin.AF_PP, af=1)
+///    pin = pyb.Pin(pyb.Pin.board.X3, mode=pyb.Pin.AF_PP, alt=1)
 
 /// \method __str__()
 /// Return a string describing the alternate function.
@@ -669,9 +668,13 @@ STATIC const mp_rom_map_elem_t pin_af_locals_dict_table[] = {
 };
 STATIC MP_DEFINE_CONST_DICT(pin_af_locals_dict, pin_af_locals_dict_table);
 
-const mp_obj_type_t pin_af_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_PinAF,
-    .print = pin_af_obj_print,
-    .locals_dict = (mp_obj_dict_t *)&pin_af_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    pin_af_type,
+    MP_QSTR_PinAF,
+    MP_TYPE_FLAG_NONE,
+    print, pin_af_obj_print,
+    locals_dict, &pin_af_locals_dict
+    );
+
+MP_REGISTER_ROOT_POINTER(mp_obj_t pin_class_mapper);
+MP_REGISTER_ROOT_POINTER(mp_obj_t pin_class_map_dict);
