@@ -83,7 +83,6 @@
 #include "i2c.h"
 #include "usb.h"
 
-extern void __fatal_error(const char *);
 #if defined(MICROPY_HW_USB_FS)
 extern PCD_HandleTypeDef pcd_fs_handle;
 #endif
@@ -192,7 +191,7 @@ void HardFault_C_Handler(ExceptionRegisters_t *regs) {
 
     /* Go to infinite loop when Hard Fault exception occurs */
     while (1) {
-        __fatal_error("HardFault");
+        MICROPY_BOARD_FATAL_ERROR("HardFault");
     }
 }
 
@@ -246,7 +245,7 @@ void NMI_Handler(void) {
 void MemManage_Handler(void) {
     /* Go to infinite loop when Memory Manage exception occurs */
     while (1) {
-        __fatal_error("MemManage");
+        MICROPY_BOARD_FATAL_ERROR("MemManage");
     }
 }
 
@@ -258,7 +257,7 @@ void MemManage_Handler(void) {
 void BusFault_Handler(void) {
     /* Go to infinite loop when Bus Fault exception occurs */
     while (1) {
-        __fatal_error("BusFault");
+        MICROPY_BOARD_FATAL_ERROR("BusFault");
     }
 }
 
@@ -270,7 +269,7 @@ void BusFault_Handler(void) {
 void UsageFault_Handler(void) {
     /* Go to infinite loop when Usage Fault exception occurs */
     while (1) {
-        __fatal_error("UsageFault");
+        MICROPY_BOARD_FATAL_ERROR("UsageFault");
     }
 }
 
@@ -297,7 +296,15 @@ void DebugMon_Handler(void) {
 /*  file (startup_stm32f4xx.s).                                               */
 /******************************************************************************/
 
-#if defined(STM32L0) || defined(STM32L432xx)
+#if defined(STM32G0)
+
+#if MICROPY_HW_USB_FS
+void USB_UCPD1_2_IRQHandler(void) {
+    HAL_PCD_IRQHandler(&pcd_fs_handle);
+}
+#endif
+
+#elif defined(STM32L0) || defined(STM32L432xx)
 
 #if MICROPY_HW_USB_FS
 void USB_IRQHandler(void) {
@@ -521,11 +528,19 @@ void ETH_WKUP_IRQHandler(void) {
 }
 #endif
 
+#if defined(STM32L1)
+void TAMPER_STAMP_IRQHandler(void) {
+    IRQ_ENTER(TAMPER_STAMP_IRQn);
+    Handle_EXTI_Irq(EXTI_RTC_TIMESTAMP);
+    IRQ_EXIT(TAMPER_STAMP_IRQn);
+}
+#else
 void TAMP_STAMP_IRQHandler(void) {
     IRQ_ENTER(TAMP_STAMP_IRQn);
     Handle_EXTI_Irq(EXTI_RTC_TIMESTAMP);
     IRQ_EXIT(TAMP_STAMP_IRQn);
 }
+#endif
 
 void RTC_WKUP_IRQHandler(void) {
     IRQ_ENTER(RTC_WKUP_IRQn);
@@ -699,6 +714,12 @@ void TIM6_DAC_LPTIM1_IRQHandler(void) {
     timer_irq_handler(6);
     IRQ_EXIT(TIM6_DAC_LPTIM1_IRQn);
 }
+#elif defined(STM32L1)
+void TIM6_IRQHandler(void) {
+    IRQ_ENTER(TIM6_IRQn);
+    timer_irq_handler(6);
+    IRQ_EXIT(TIM6_IRQn);
+}
 #else
 void TIM6_DAC_IRQHandler(void) {
     IRQ_ENTER(TIM6_DAC_IRQn);
@@ -765,6 +786,26 @@ void TIM8_TRG_COM_TIM14_IRQHandler(void) {
 }
 #endif
 
+#if defined(STM32L1)
+void TIM9_IRQHandler(void) {
+    IRQ_ENTER(TIM9_IRQn);
+    timer_irq_handler(9);
+    IRQ_EXIT(TIM9_IRQn);
+}
+
+void TIM10_IRQHandler(void) {
+    IRQ_ENTER(TIM9_IRQn);
+    timer_irq_handler(10);
+    IRQ_EXIT(TIM9_IRQn);
+}
+
+void TIM11_IRQHandler(void) {
+    IRQ_ENTER(TIM9_IRQn);
+    timer_irq_handler(11);
+    IRQ_EXIT(TIM9_IRQn);
+}
+#endif
+
 #if defined(STM32G0)
 void TIM14_IRQHandler(void) {
     IRQ_ENTER(TIM14_IRQn);
