@@ -143,14 +143,20 @@ STATIC void mp_help_print_obj(const mp_obj_t obj) {
         if (type == &mp_type_type) {
             type = MP_OBJ_TO_PTR(obj);
         }
-        if (type->locals_dict != NULL) {
-            map = &type->locals_dict->map;
+        if (MP_OBJ_TYPE_HAS_SLOT(type, locals_dict)) {
+            map = &MP_OBJ_TYPE_GET_SLOT(type, locals_dict)->map;
         }
     }
     if (map != NULL) {
         for (uint i = 0; i < map->alloc; i++) {
-            if (map->table[i].key != MP_OBJ_NULL) {
-                mp_help_print_info_about_object(map->table[i].key, map->table[i].value);
+            mp_obj_t key = map->table[i].key;
+            if (key != MP_OBJ_NULL
+                #if MICROPY_MODULE_ATTR_DELEGATION
+                // MP_MODULE_ATTR_DELEGATION_ENTRY entries have MP_QSTRnull as qstr key.
+                && key != MP_OBJ_NEW_QSTR(MP_QSTRnull)
+                #endif
+                ) {
+                mp_help_print_info_about_object(key, map->table[i].value);
             }
         }
     }
