@@ -71,12 +71,11 @@ typedef struct _mp_sched_item_t {
     mp_obj_t arg;
 } mp_sched_item_t;
 
-// This structure hold information about the memory allocation system.
-typedef struct _mp_state_mem_t {
-    #if MICROPY_MEM_STATS
-    size_t total_bytes_allocated;
-    size_t current_bytes_allocated;
-    size_t peak_bytes_allocated;
+// This structure holds information about a single contiguous area of
+// memory reserved for the memory manager.
+typedef struct _mp_state_mem_area_t {
+    #if MICROPY_GC_SPLIT_HEAP
+    struct _mp_state_mem_area_t *next;
     #endif
 
     byte *gc_alloc_table_start;
@@ -87,8 +86,25 @@ typedef struct _mp_state_mem_t {
     byte *gc_pool_start;
     byte *gc_pool_end;
 
+    size_t gc_last_free_atb_index;
+} mp_state_mem_area_t;
+
+// This structure hold information about the memory allocation system.
+typedef struct _mp_state_mem_t {
+    #if MICROPY_MEM_STATS
+    size_t total_bytes_allocated;
+    size_t current_bytes_allocated;
+    size_t peak_bytes_allocated;
+    #endif
+
+    mp_state_mem_area_t area;
+
     int gc_stack_overflow;
-    MICROPY_GC_STACK_ENTRY_TYPE gc_stack[MICROPY_ALLOC_GC_STACK_SIZE];
+    MICROPY_GC_STACK_ENTRY_TYPE gc_block_stack[MICROPY_ALLOC_GC_STACK_SIZE];
+    #if MICROPY_GC_SPLIT_HEAP
+    // Array that tracks the area for each block on gc_block_stack.
+    mp_state_mem_area_t *gc_area_stack[MICROPY_ALLOC_GC_STACK_SIZE];
+    #endif
 
     // This variable controls auto garbage collection.  If set to 0 then the
     // GC won't automatically run when gc_alloc can't find enough blocks.  But
@@ -100,7 +116,9 @@ typedef struct _mp_state_mem_t {
     size_t gc_alloc_threshold;
     #endif
 
-    size_t gc_last_free_atb_index;
+    #if MICROPY_GC_SPLIT_HEAP
+    mp_state_mem_area_t *gc_last_free_area;
+    #endif
 
     #if MICROPY_PY_GC_COLLECT_RETVAL
     size_t gc_collected;
@@ -150,69 +168,19 @@ typedef struct _mp_state_vm_t {
     // dictionary with loaded modules (may be exposed as sys.modules)
     mp_obj_dict_t mp_loaded_modules_dict;
 
-    #if MICROPY_ENABLE_SCHEDULER
-    mp_sched_item_t sched_queue[MICROPY_SCHEDULER_DEPTH];
-    #endif
-
-    // current exception being handled, for sys.exc_info()
-    #if MICROPY_PY_SYS_EXC_INFO
-    mp_obj_base_t *cur_exception;
-    #endif
-
-    #if MICROPY_PY_SYS_ATEXIT
-    // exposed through sys.atexit function
-    mp_obj_t sys_exitfunc;
-    #endif
-
     // dictionary for the __main__ module
     mp_obj_dict_t dict_main;
 
-    #if MICROPY_PY_SYS
-    // If MICROPY_PY_SYS_PATH_ARGV_DEFAULTS is not enabled then these two lists
-    // must be initialised after the call to mp_init.
-    mp_obj_list_t mp_sys_path_obj;
-    mp_obj_list_t mp_sys_argv_obj;
-
-    #if MICROPY_PY_SYS_ATTR_DELEGATION
-    // Contains mutable sys attributes.
-    mp_obj_t sys_mutable[MP_SYS_MUTABLE_NUM];
-    #endif
-    #endif
-
     // dictionary for overridden builtins
     #if MICROPY_CAN_OVERRIDE_BUILTINS
     mp_obj_dict_t *mp_module_builtins_override_dict;
     #endif
 
-    #if MICROPY_PERSISTENT_CODE_TRACK_RELOC_CODE
-    // An mp_obj_list_t that tracks relocated native code to prevent the GC from reclaiming them.
-    mp_obj_t track_reloc_code_list;
-    #endif
-
-    // include any root pointers defined by a port
-    MICROPY_PORT_ROOT_POINTERS
-
-    // root pointers for extmod
-
-    #if MICROPY_REPL_EVENT_DRIVEN
-    vstr_t *repl_line;
-    #endif
-
-    #if MICROPY_PY_OS_DUPTERM
-    mp_obj_t dupterm_objs[MICROPY_PY_OS_DUPTERM];
-    #endif
-
-    #if MICROPY_PY_LWIP_SLIP
-    mp_obj_t lwip_slip_stream;
-    #endif
-
-    #if MICROPY_VFS
-    struct _mp_vfs_mount_t *vfs_cur;
-    struct _mp_vfs_mount_t *vfs_mount_table;
-    #endif
-
-    #if MICROPY_PY_BLUETOOTH
-    mp_obj_t bluetooth;
+    // Include any root pointers registered with MP_REGISTER_ROOT_POINTER().
+    #ifndef NO_QSTR
+    // Only include root pointer definitions when not doing qstr extraction, because
+    // the qstr extraction stage also generates the root pointers header file.
+    #include "genhdr/root_pointers.h"
     #endif
 
     //
@@ -258,6 +226,11 @@ typedef struct _mp_state_vm_t {
     uint8_t sched_idx;
     #endif
 
+    #if MICROPY_ENABLE_VM_ABORT
+    bool vm_abort;
+    nlr_buf_t *nlr_abort;
+    #endif
+
     #if MICROPY_PY_THREAD_GIL
     // This is a global mutex used to make the VM/runtime thread-safe.
     mp_thread_mutex_t gil_mutex;
@@ -329,8 +302,10 @@ extern mp_state_ctx_t mp_state_ctx;
 #if MICROPY_PY_THREAD
 extern mp_state_thread_t *mp_thread_get_state(void);
 #define MP_STATE_THREAD(x) (mp_thread_get_state()->x)
+#define mp_thread_is_main_thread() (mp_thread_get_state() == &mp_state_ctx.thread)
 #else
 #define MP_STATE_THREAD(x)  MP_STATE_MAIN_THREAD(x)
+#define mp_thread_is_main_thread() (true)
 #endif
 
 #endif // MICROPY_INCLUDED_PY_MPSTATE_H
