@@ -202,12 +202,16 @@ STATIC mp_obj_t load_obj(mp_reader_t *reader) {
         read_bytes(reader, (byte *)vstr.buf, len);
         if (obj_type == MP_PERSISTENT_OBJ_STR || obj_type == MP_PERSISTENT_OBJ_BYTES) {
             read_byte(reader); // skip null terminator
-            return mp_obj_new_str_from_vstr(obj_type == MP_PERSISTENT_OBJ_STR ? &mp_type_str : &mp_type_bytes, &vstr);
+            if (obj_type == MP_PERSISTENT_OBJ_STR) {
+                return mp_obj_new_str_from_utf8_vstr(&vstr);
+            } else {
+                return mp_obj_new_bytes_from_vstr(&vstr);
+            }
         } else if (obj_type == MP_PERSISTENT_OBJ_INT) {
             return mp_parse_num_integer(vstr.buf, vstr.len, 10, NULL);
         } else {
             assert(obj_type == MP_PERSISTENT_OBJ_FLOAT || obj_type == MP_PERSISTENT_OBJ_COMPLEX);
-            return mp_parse_num_decimal(vstr.buf, vstr.len, obj_type == MP_PERSISTENT_OBJ_COMPLEX, false, NULL);
+            return mp_parse_num_float(vstr.buf, vstr.len, obj_type == MP_PERSISTENT_OBJ_COMPLEX, NULL);
         }
     }
 }
@@ -386,64 +390,66 @@ STATIC mp_raw_code_t *load_raw_code(mp_reader_t *reader, mp_module_context_t *co
     return rc;
 }
 
-mp_compiled_module_t mp_raw_code_load(mp_reader_t *reader, mp_module_context_t *context) {
+void mp_raw_code_load(mp_reader_t *reader, mp_compiled_module_t *cm) {
     byte header[4];
     read_bytes(reader, header, sizeof(header));
+    byte arch = MPY_FEATURE_DECODE_ARCH(header[2]);
     if (header[0] != 'M'
         || header[1] != MPY_VERSION
-        || MPY_FEATURE_DECODE_FLAGS(header[2]) != MPY_FEATURE_FLAGS
+        || (arch != MP_NATIVE_ARCH_NONE && MPY_FEATURE_DECODE_SUB_VERSION(header[2]) != MPY_SUB_VERSION)
         || header[3] > MP_SMALL_INT_BITS) {
         mp_raise_ValueError(MP_ERROR_TEXT("incompatible .mpy file"));
     }
     if (MPY_FEATURE_DECODE_ARCH(header[2]) != MP_NATIVE_ARCH_NONE) {
-        byte arch = MPY_FEATURE_DECODE_ARCH(header[2]);
         if (!MPY_FEATURE_ARCH_TEST(arch)) {
-            mp_raise_ValueError(MP_ERROR_TEXT("incompatible .mpy arch"));
+            if (MPY_FEATURE_ARCH_TEST(MP_NATIVE_ARCH_NONE)) {
+                // On supported ports this can be resolved by enabling feature, eg
+                // mpconfigboard.h: MICROPY_EMIT_THUMB (1)
+                mp_raise_ValueError(MP_ERROR_TEXT("native code in .mpy unsupported"));
+            } else {
+                mp_raise_ValueError(MP_ERROR_TEXT("incompatible .mpy arch"));
+            }
         }
     }
 
     size_t n_qstr = read_uint(reader);
     size_t n_obj = read_uint(reader);
-    mp_module_context_alloc_tables(context, n_qstr, n_obj);
+    mp_module_context_alloc_tables(cm->context, n_qstr, n_obj);
 
     // Load qstrs.
     for (size_t i = 0; i < n_qstr; ++i) {
-        context->constants.qstr_table[i] = load_qstr(reader);
+        cm->context->constants.qstr_table[i] = load_qstr(reader);
     }
 
     // Load constant objects.
     for (size_t i = 0; i < n_obj; ++i) {
-        context->constants.obj_table[i] = load_obj(reader);
+        cm->context->constants.obj_table[i] = load_obj(reader);
     }
 
     // Load top-level module.
-    mp_compiled_module_t cm2;
-    cm2.rc = load_raw_code(reader, context);
-    cm2.context = context;
+    cm->rc = load_raw_code(reader, cm->context);
 
     #if MICROPY_PERSISTENT_CODE_SAVE
-    cm2.has_native = MPY_FEATURE_DECODE_ARCH(header[2]) != MP_NATIVE_ARCH_NONE;
-    cm2.n_qstr = n_qstr;
-    cm2.n_obj = n_obj;
+    cm->has_native = MPY_FEATURE_DECODE_ARCH(header[2]) != MP_NATIVE_ARCH_NONE;
+    cm->n_qstr = n_qstr;
+    cm->n_obj = n_obj;
     #endif
 
     reader->close(reader->data);
-
-    return cm2;
 }
 
-mp_compiled_module_t mp_raw_code_load_mem(const byte *buf, size_t len, mp_module_context_t *context) {
+void mp_raw_code_load_mem(const byte *buf, size_t len, mp_compiled_module_t *context) {
     mp_reader_t reader;
     mp_reader_new_mem(&reader, buf, len, 0);
-    return mp_raw_code_load(&reader, context);
+    mp_raw_code_load(&reader, context);
 }
 
 #if MICROPY_HAS_FILE_READER
 
-mp_compiled_module_t mp_raw_code_load_file(const char *filename, mp_module_context_t *context) {
+void mp_raw_code_load_file(const char *filename, mp_compiled_module_t *context) {
     mp_reader_t reader;
     mp_reader_new_file(&reader, filename);
-    return mp_raw_code_load(&reader, context);
+    mp_raw_code_load(&reader, context);
 }
 
 #endif // MICROPY_HAS_FILE_READER
@@ -581,21 +587,18 @@ void mp_raw_code_save(mp_compiled_module_t *cm, mp_print_t *print) {
     // header contains:
     //  byte  'M'
     //  byte  version
-    //  byte  feature flags
+    //  byte  native arch (and sub-version if native)
     //  byte  number of bits in a small int
     byte header[4] = {
         'M',
         MPY_VERSION,
-        MPY_FEATURE_ENCODE_FLAGS(MPY_FEATURE_FLAGS_DYNAMIC),
+        cm->has_native ? MPY_FEATURE_ENCODE_SUB_VERSION(MPY_SUB_VERSION) | MPY_FEATURE_ENCODE_ARCH(MPY_FEATURE_ARCH_DYNAMIC) : 0,
         #if MICROPY_DYNAMIC_COMPILER
         mp_dynamic_compiler.small_int_bits,
         #else
         MP_SMALL_INT_BITS,
         #endif
     };
-    if (cm->has_native) {
-        header[2] |= MPY_FEATURE_ENCODE_ARCH(MPY_FEATURE_ARCH_DYNAMIC);
-    }
     mp_print_bytes(print, header, sizeof(header));
 
     // Number of entries in constant table.
@@ -634,6 +637,9 @@ void mp_raw_code_save_file(mp_compiled_module_t *cm, const char *filename) {
     MP_THREAD_GIL_EXIT();
     int fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0644);
     MP_THREAD_GIL_ENTER();
+    if (fd < 0) {
+        mp_raise_OSError_with_filename(errno, filename);
+    }
     mp_print_t fd_print = {(void *)(intptr_t)fd, fd_print_strn};
     mp_raw_code_save(cm, &fd_print);
     MP_THREAD_GIL_EXIT();
@@ -644,3 +650,8 @@ void mp_raw_code_save_file(mp_compiled_module_t *cm, const char *filename) {
 #endif // MICROPY_PERSISTENT_CODE_SAVE_FILE
 
 #endif // MICROPY_PERSISTENT_CODE_SAVE
+
+#if MICROPY_PERSISTENT_CODE_TRACK_RELOC_CODE
+// An mp_obj_list_t that tracks relocated native code to prevent the GC from reclaiming them.
+MP_REGISTER_ROOT_POINTER(mp_obj_t track_reloc_code_list);
+#endif
